(* ::Package:: *)

(* ::Title:: *)
(*SDA package*)


(* ::Subtitle:: *)
(*Version 1-01 updated 3/12/18*)


(*
Purpose 
This Mathematica package contains all routines to perform the Loop Eigenvalue Elasticity Analysis (LEEA) and the DDWA as described in Kampmann and Oliva (2006).

Requirements
The package requires Mathematica 11.0 or higher. The mdlDash vxx.nb and mdlLEEA vxx.wl files have to be located in the same directory.

Input files
1. Vensim model as ( *.mdl ) file that contains the continuously differentiable model (equations do not contain IF THEN ELSE, MIN, MAX, etc.)
2. A tab delimited file with the model simulation output if the model refers to the external data file.  
Use Vensim Model>Dataset Export ... command (with options "tab" and "time running across") to translate the Vensim data file ( *.vdf ) into a tab delimited file ( *.tab ).  

	
Instructions
1. Open (File>Open ...) the notebook file ( mdlDash vxx.nb) that contains the main dashboard of the utiliy and agree to automatically evaluate the initialization cells.
2. Load the model from the dashboard.  Once the file is loaded, the utility will proceed to do the computations. The utility provides interactive reporting capabilities in multiple formats. 
   All output can be copied or exported to incorporate in spreadsheets or word processing software.    


Limitations
1. This implementation of LEEA works only with continuous (C1) functions.  Please ensure that the Vensim model that is imported into Mathematica adheres to this requirement
2. Pay attention to the CamelCase functions of Mathematica (ArcSin needs to be entered in Vensim exctly like that to ensure flawless import).

History
v0.1	02/2006		Core functionality with explicit model and data inputs.
v0.2	02/2006		Eliminated the link elasticity report and included the influence computations.
v0.3	11/2012		Upgraded to Mathematica 8.0 compatability.
v0.4	02/2013		Check compatability with Mathematica 9.0. Incorporated all ploting and reporting to the notebook. Improvements in computation speed.
v0.5	04/2014		Updated to work with ModelVariables` Context and data variables. improvements on Loop Analysis Displays.
v0.6	02/2015		Updated Mathematica 10.0. Added Loop Influece over time graph.	
v1.0	04/2017		Interface, built-in parser of .mdl files

mdlLEEA Versions
v1.01	04/2017		LEEA package as a function
v1.05	04/2017		Built-in Euler integration
v1.09	05/2017		Dashboard interface
v2.02	06/2017		Single package containing all the functionality 
v2.09	07/2017		Full integration of LEEA and DDWA

SDA Versions
v1.00	09/2017		Launched as a tool
v1.01	03/2018		Updated DDWA analysis

*)


(* ::Section:: *)
(*Package begins*)


BeginPackage["mdlLEEA`"];


mdlDash;


Begin["`Private`"];



(* ::Section:: *)
(*FeedbackLoops functions*)


(* ::Subsection:: *)
(*Help on the functions*)


FBBinaryQ::usage = "FBBinaryQ[x] tests if x belongs to the binary field {0,1}."
FBShortestPath::usage = "FBShortestPath[a, start, end] finds a shortest path between vertices start and end in the graph represented by the adjacency list a"
FBAdjMatrixQ::usage = "FBAdjMatrixQ[m] tests if m is an adjacency matrix."
FBToEdgeList::usage = "FBToEdgeList[m] constructs a list of pairs {x,y} of the edges from the adjacency matrix m."
FBAdjacencyMatrix::usage = "FBAdjacencyMatrix[e] constructs the adjacency matrix from the list e of edges {x,y}.  FBToEdgeList[c] or FBToEdgeList[{c1,c2,...}] converts the cycle c (or cycles c1,c2,...) to edge list(s)."
FBToAdjacencyLists::usage = "FBToAdjacencyLists[m] constructs an adjacency list representation of the adjacency matrix m."
FBFromAdjacencyLists::usage = "FBFromAdjacencyLists[a] constructs the adjacency matrix from the adjacency list a."
FBStrongComponents::usage = "FBStrongComponents[m] returns the strongly connected components of the graph with adjacency matrix m."
FBConnectedQ::usage = "FBConnectedQ[m] tests whether the adjacency matrix m is strongly connected."
FBDirectedCycles::usage = "FBDirectedCycles[m] produces a list of all directed cycles (as a vertex list {x,y,z,..}) in the graph with adjacency matrix m."
FBCycleMatrix::usage = "FBCycleMatrix[c,e] returns a (0,1) matrix where the (i,j)th element is 1 if cycle i in the cycle list c contains edge j in the edge list e."
FBIncidenceMatrix::usage = "FBIncidenceMatrix[m] produces the (0,1,-1) incidence matrix where the (i,j)th element is 1 if edge j originates in node i, -1 if it ends in node i, and 0 otherwise."
FBRank::usage = "FBRank[A] finds the rank of the matrix A."
FBRemoveDependentRows::usage = "FBRemoveDependentRows[A] finds FBRank[A] linearly independent rows of A."
FBRemoveDependentColumns::usage = "FBRemoveDependentColumns[A] finds FBRank[A] linearly independent columns of A."
FBKernel::usage = "FBKernel[A] returns a non-singular FBRank[A]-square submatrix of A."
FBBRowReduce::usage = "FBBRowReduce[A] gives the row-reduced form of the binary-field matrix A."
FBBDet::usage = "FBBDet[A] returns the determinant of the binary-field square matrix A."
FBBNullSpace::usage = "FBBNullSpace[A] returns the null space of the binary-field matrix A."
FBBDot::usage = "FBBDot[a,b] gives the product of the binary-field vectors or matrices a,b."
FBCycleGain::usage = "FBCycleGain[c,A] returns the gain of the cycle c based on the system matrix A.  The empty cycle {} by definitino has gain 1.  FBCycleGain[{c1,c2,...},A] returns a list of gains for each cycle c1,c2,..."
FBCompactPrint::usage = "FBCompactPrint[A] prints the matrix A with no spacing between elements."
FBSILS::usage = "FBSILS[m] returns the Shortest Independent Loop Set (SILS) found by the SILS algorithm by R. Oliva" 
FBGeodeticCycles::usage = "FBGeodeticCycles[m] produces a list of all geodetic directed cycles (as vertex lists {x,y,z,..}) in the graph with adjacency matrix m."
FBIndependentLoopSet::usage = "FBIndependentLoopSet[g] returns a set of independent loops for the graph with edgelist e, using the algorithm developed by Kampmann 1996/2012.  Returns a list 
	{{\!\(\*SubscriptBox[\(l\), \(1\)]\), ..., \!\(\*SubscriptBox[\(l\), \(k\)]\)},{\!\(\*SubscriptBox[\(e\), \(1\)]\), ..., \!\(\*SubscriptBox[\(e\), \(k\)]\)}}, where \!\(\*SubscriptBox[\(l\), \(i\)]\) is the node sequence of loop i and \!\(\*SubscriptBox[\(e\), \(i\)]\) is the edge that was used to first construct that loop. (The edges \!\(\*SubscriptBox[\(e\), \(1\)]\), ..., \!\(\*SubscriptBox[\(e\), \(k\)]\) 
	can be used to calculate the loop elasticies from the edge elasticites by backward substitution.)";
    (* CEK: There may be an issue here whether one can properly distinguish edge lists and adjacency
    lists.  Thus, I have chosen to make it a requirement that the function be given the edge list.
  It is easy to extend it to also take an adjacency matrix or an adjacency list as input. *)


(* ::Subsubsection:: *)
(*Utility routines to determine structure of matrix and vectors*)


FBBinaryQ[x_] :=
    MatchQ[x,0|1]
FBAdjMatrixQ[m_] :=
    MatchQ[Dimensions[m],{d_,d_}]&&MatrixQ[m,FBBinaryQ]
FBEdgeListQ[e_] :=
    MatrixQ[e,(IntegerQ[#]&&Positive[#])&]&&Length[e[[1]]]==2
FBCycleQ[c_] :=
    VectorQ[c,(IntegerQ[#]&&Positive[#])&]
FBCycleQ[{}] :=
    True (*empty cycle*)
FBCycleListQ[c_] :=
    ListQ[c]&&Apply[And,Map[FBCycleQ,c]]


(* ::Subsubsection:: *)
(*Misc. functions from combinatorica.m (sometimes modified)*)


FBPermutationQ[p_List] :=
    (Sort[p] == Range[Length[p]])


FBToCycles[p1_?FBPermutationQ] :=
    Module[ {p = p1,m,n,cycle,i},
        Select[
            Table[
                m = n = p[[i]];
                cycle = {};
                While[ p[[n]] != 0,
                    AppendTo[cycle,m = n];
                    n = p[[n]];
                    p[[m]] = 0 
                ];
                cycle,
                {i,Length[p]}
            ],
            (# =!= {})&
        ]
    ]


FBToEdgeList[A_?FBAdjMatrixQ] :=
    Position[ A, _?(Function[n,n != 0]) ]
FBToEdgeList[c_?FBCycleQ] :=
    Partition[Append[c,First[c]],2,1]
FBToEdgeList[c_?FBCycleListQ] :=
    Map[FBToEdgeList,c]
FBAdjacencyMatrix[l_?FBEdgeListQ] :=
    Module[ {n = Max[l]},
        MapAt[1&, Table[0,{n},{n}],l]
    ]


FBToAdjacencyLists[A_] :=
    Map[ (Flatten[ Position[ #, _?(Function[n, n!=0])] ])&, A ]/;FBAdjMatrixQ[A]


FBFromAdjacencyLists[e_List] :=
    Module[ {blanks = Table[0,{Length[e]}] },
        Map [ (MapAt[ 1&,blanks,Partition[#,1]])&, e ]
    ]


FBStrongComponents[A_?FBAdjMatrixQ] :=
    Block[ {e = FBToAdjacencyLists[A],s,c = 1,i,cur = {},low = dfs = Table[0,{Length[A]}],scc = {}},
        While[(s = Select[Range[Length[A]],(dfs[[#]]==0)&]) != {},
            FBSearchStrongComp[First[s]];
        ];
        scc
    ]


FBSearchStrongComp[v_Integer] :=
    Block[ {r},
        low[[v]] = dfs[[v]] = c++;
        PrependTo[cur,v];
        Scan[
            (If[ dfs[[#]] == 0,
                 FBSearchStrongComp[#];
                 low[[v]] = Min[low[[v]],low[[#]]],
                 If[ (dfs[[#]] < dfs[[v]]) && MemberQ[cur,#],
                     low[[v]] = Min[low[[v]],dfs[[#]] ]
                 ];
             ])&,
            e[[v]]
        ];
        If[ low[[v]] == dfs[[v]],
            {r} = Flatten[Position[cur,v]];
            AppendTo[scc,Take[cur,r]];
            cur = Drop[cur,r];
        ];
    ]


FBConnectedQ[A_?FBAdjMatrixQ] :=
    Length[ FBStrongComponents[A] ] == 1


(* ::Subsubsection:: *)
(*Directed cycles, incidence vectors and matrices *)


FBDirectedCycles[m_?FBAdjMatrixQ] :=
    Block[ {i,ll = {},a = FBToAdjacencyLists[m]},
        For[i = 1,i<=Length[m],a = Map[DeleteCases[#,_?(Function[n,n==i])]&,a];
                               i++,Map[FBFDC[{},a,i,#]&,a[[i]]]];
        Sort[Map[FBCyclicSort,ll]]
    ]
FBFDC[p_List,al_List,x0_Integer,x1_Integer] :=
    Block[ {nn,pn},
        If[ x0==x1,
            AppendTo[ll,Append[p,x0]],
            pn = Append[p,x1];
            nn = Complement[al[[x1]],pn];
            Map[FBFDC[pn,al,x0,#]&,nn]
        ];
    ]


FBCycleMatrix[c_?FBCycleListQ,e_?FBEdgeListQ] :=
    Block[ {l = FBToEdgeList[c]},
        Map[Function[x,Map[If[ FreeQ[x,#],
                               0,
                               1
                           ]&,e]],l]
    ]


FBIncidenceMatrix[A_?FBAdjMatrixQ] :=
    Block[ {n = Length[A]},
        Transpose[Map[(Join[Table[0,{First[#]-1}],{1},Table[0,{n-First[#]}]]-Join[Table[0,{Last[#]-1}],{1},Table[0,{n-Last[#]}]])&,
        FBToEdgeList[A]
        ]
        ]
    ]


(* ::Subsubsection:: *)
(*Real-field linear algebra*)


FBRank[A_?MatrixQ] :=
    Module[ {d = Dimensions[A]},
        If[ d[[1]]<d[[2]],
            d[[1]]-Length[NullSpace[Transpose[A]]],
            d[[2]]-Length[NullSpace[A]]
        ]
    ]
FBRank[{}] = 0;
FBRank[{{}}] = 0;


FBRemoveDependentRows[A_?MatrixQ] :=
    Module[ {r,m,i,ns,sacred,nr},
        nr = Length[A];
        ns = NullSpace[Transpose[A]];
        If[ ns==={},
            Return[A]
        ];
        sacred = Flatten[Position[Map[(Count[#,_?(UnsameQ[#,0]&)]==1)&,Transpose[ns]],False]];
        For[i = 1;
            r = {},i<=Length[ns],i++,For[j = 1,j<=Length[ns[[i]]],j++,If[ MemberQ[sacred,j]||MemberQ[r,j]||ns[[i,j]]===0,
                                                                          Continue[]
                                                                      ];
                                                                      AppendTo[r,j];
                                                                      Break[];]] For[i = 1;
                                                                                     m = {},i<=nr,i++,If[ FreeQ[r,i],
                                                                                                          AppendTo[m,A[[i]]]
                                                                                                      ];];
        m
    ]


FBRemoveDependentColumns[A_?MatrixQ] :=
    Transpose[FBRemoveDependentRows[Transpose[A]]]


FBKernel[A_?MatrixQ] :=
    FBRemoveDependentColumns[FBRemoveDependentRows[A]]


(* ::Subsubsection:: *)
(*Binary-field linear algebra*)


SetAttributes[FBBTimes,{Flat,Orderless,OneIdentity,Listable}]
FBBTimes[1,1] :=
    1
FBBTimes[0,1] :=
    0
FBBTimes[0,0] :=
    0
FBBTimes[-1,x_] :=
    x


SetAttributes[FBPlus,{Flat,Orderless,OneIdentity,Listable}];
FBPlus[1,1] :=
    0;
FBPlus[0,1] :=
    1;
FBPlus[0,0] :=
    0;
FBPlus[-1,x_] :=
    FBPlus[1,x];


FBBDet[A_] :=
    Module[ {i,B = Sort[A,(#1[[1]]>#2[[1]])&]},
        If[ B[[1,1]]==0,
            Return[0]
        ];
        For[i = 2,i<=Length[B],i++,If[ B[[i,1]]!=0,
                                       B[[i]] = FBPlus[B[[i]],B[[1]]]
                                   ];];
        Return[FBBDet[Rest[Transpose[Rest[Transpose[B]]]]]];
    ]/;MatrixQ[A,FBBinaryQ]&&Length[A]==Length[A[[1]]]
FBBDet[{{x_?FBBinaryQ}}] :=
    x


FBBNullSpace[A_] :=
    Module[ {B,m,n,ol,al,ns,i,j,k,r},
        B = A/.{i_:>1/;i!=0};
        m = Length[B];
        n = Length[B[[1]]];
        ol = {};
        al = Range[n];
        For[r = 1;
            i = 1,r<=m&&i<=n,For[j = r,j<=m&&B[[j,i]]==0,j++];
                             If[ j<=m,
                                 If[ B[[r,i]]==0,
                                     B[[r]] = FBPlus[B[[r]],B[[j]]];
                                 ];
                                 For[k = 1,k<=m,k++,If[ k!=r&&B[[k,i]]!=0,
                                                        B[[k]] = FBPlus[B[[k]],B[[r]]]
                                                    ]];
                                 AppendTo[ol,i];
                                 r++;
                             ];
                             i++;];
        r = Length[ol];
        al = Complement[al,ol];
        If[ r<m,
            B = Drop[B,{r+1,m}]
        ];
        For[i = 1;
            ns = Table[0,{n-r},{n}],i<=(n-r),i++,ns[[i,al[[i]]]] = 1;
                                                 For[j = 1,j<=r,j++,ns[[i,ol[[j]]]] = B[[j,al[[i]]]]];];
        ns
    ]/;MatrixQ[A,FBBinaryQ]


FBBRank[A_] :=
    Module[ {d = Dimensions[A]},
        If[ d[[1]]<d[[2]],
            d[[1]]-Length[FBBNullSpace[Transpose[A]]],
            d[[2]]-Length[FBBNullSpace[A]]
        ]
    ]/;MatrixQ[A,FBBinaryQ]


FBBDot[A_,B_] :=
    Inner[FBBTimes,A,B,FBPlus]/;MatrixQ[A,FBBinaryQ]&&MatrixQ[B,FBBinaryQ]


FBBRowReduce[A_] :=
    Module[ {B,m,n,i,j,k,r},
        B = A/.{i_:>1/;i!=0};
        m = Length[B];
        n = Length[B[[1]]];
        For[r = 1;
            i = 1,r<=m&&i<=n,For[j = r,j<=m&&B[[j,i]]==0,j++];
                             If[ j<=m,
                                 If[ B[[r,i]]==0,
                                     B[[r]] = FBPlus[B[[r]],B[[j]]];
                                 ];
                                 For[k = 1,k<=m,k++,If[ k!=r&&B[[k,i]]!=0,
                                                        B[[k]] = FBPlus[B[[k]],B[[r]]]
                                                    ]];
                                 r++;
                             ];
                             i++;];
        B
    ]/;MatrixQ[A,FBBinaryQ]


(* ::Subsubsection:: *)
(*Misc. utility routines*)


FBCyclicSort[l_List] :=
    RotateLeft[l,First[First[Position[l,First[Sort[l]]]]]-1]
FBCyclicSort[{}] :=
    {}


FBRemoveSelfLoops[m_?FBAdjMatrixQ] :=
    Table[If[ i==j,
              0,
              m[[i,j]]
          ],{i,Length[m]},{j,Length[m]}]
FBRemoveSelfLoops[e_?FBEdgeListQ] :=
    DeleteCases[e,{i_,i_}]


FBCycleGain[c_List,A_?MatrixQ] :=
    Module[ {c2 = Map[Reverse,Partition[Append[c,c[[1]]],2,1]]},
        Apply[Times,Map[Function[x,Apply[Part[A,#1,#2]&,x]],c2]]
    ]/;FBCycleGain[{},_] :=
        1
FBCycleGain[c_?FBCycleListQ,A_?MatrixQ] :=
    Map[FBCycleGain[#,A]&,c]


FBCompactPrint[A_?MatrixQ] :=
    Scan[Print[StringJoin[Map[ToString,#]]]&,A]


(* ::Subsubsection:: *)
(*Shortest independent loop set (FBSILS)*)


BinSum[a_?FBAdjMatrixQ,b_?FBAdjMatrixQ] :=
    Map[Clip[#,{0,1}]&,Plus[a,b],{2}]
BinSub[a_?FBAdjMatrixQ,b_?FBAdjMatrixQ] :=
    Map[Clip[#,{0,1}]&,Plus[a,-b],{2}]
BinProd[a_?FBAdjMatrixQ,b_?FBAdjMatrixQ] :=
    Map[Clip[#,{0,1}]&,Dot[a,b],{2}]

VertexDistance[a_?FBAdjMatrixQ] :=
    Module[ {A,G,p,s,i = 2},
        A = a+IdentityMatrix[Length[a]];
        G = A-IdentityMatrix[Length[A]];
        p = A;
        s = BinProd[A,p];
        While[p=!=s,
            G = G+i*(s-p);
            p = s;
            i++;
            s = BinProd[A,p];
        ];
        G
    ] 

TrackVertex[u_?IntegerQ, v_?IntegerQ,dist_?MatrixQ] :=
    Module[ {j, loop, rh, lh},
        loop = {u};(*the u->v path; enter first element in loop list*)
        For [j = 1,j<=(dist[[u,v]]-1), j++,(*for 1 to length(u->v)-1*)
            rh = Position[dist[[All,v]],dist[[u,v]]-j]; (*pred of v with step= length(u->v)-j*)
            lh = Position[dist[[Last[loop],All]],1];  (*succ of last element in loop with step=1*)
            AppendTo[loop,Intersection[rh,lh][[1]][[1]]]; (*Append to loop the first common element of rh&lh*)
        ];
        AppendTo[loop,v]; (*the v->u path; append v*)
        For [j = 1,j<=(dist[[v,u]]-1),j++, (*for 1 to length(v->u)-1*)
            rh = Position[dist[[All,u]],dist[[v,u]]-j]; (*pred of u with step=length(v->u)-j*)
            lh = Position[dist[[Last[loop],All]],1]; (*succ of last element in loop with step=1*)
            AppendTo[loop,Intersection[rh,lh][[1]][[1]]]; (*Append to loop the first common element of rh&lh*)
        ];
        loop = RotateLeft[loop,Position[loop,Min[loop]][[1]][[1]]-1]; (*rotate loop to start w/ smallest node #*)
        loop
    (*Return loop*)
    ]

FBGeodeticCycles[A_?FBAdjMatrixQ] :=
    Module[ {sc,index,d,L,loops = {},i,j,k,pairs,loop},
        sc = FBStrongComponents[A]; (*identify strong components*)
        For[k = 1,k<=Length[sc],k++,(*for each strong component*)
            If[ Length[sc[[k]]]>1,(*if more than one element*)
                index = Sort[sc[[k]]]; (*index to extract sc from AdjM*)
                d = VertexDistance[A[[index,index]]]; (*calculate distance among vertex*)
                L = Transpose[UpperTriangularize[d]]+LowerTriangularize[d]; (*estimate loop length by adding both sides (u->v+v->u)*)
                For[i = 2,i<=Max[L],i++,(*for all potential loop sizes*)
                    pairs = Position[L,i];(*id pairs of terminal vertex of loops length i*)
                    For[j = 1,j<=Length[pairs],j++,(*for all pairs*)
                        loop = TrackVertex[pairs[[j,1]],pairs[[j,2]],d];
                        If[ Length[loop]==Length[Union[loop]],(*If loop not figure 8*)
                            loop = Map[Extract[index,#]&,loop,{1}];(*change back to index # s*)
                            AppendTo[loops,loop];
                        ] (*if*)
                    ] (* next pair *)
                ]
            (* next loop size *)
            ] (* end if more than one element *)
        ]; (* next strong component *)
        loops = Union[loops]; (*eliminate duplicate loops*)
        loops
    (* return loop set *)
    ]

FBSILS[input_] :=
    Module[ {a,B,Edges,LA,c, i,S = {}},
        a = input;
        B = SparseArray[{},{Max[a],Max[a]}];(*create empty base AdjMat*)
        Edges = Map[Append[Partition[#,2,1],List[Last[#],First[#]]]&,a,{1}]; 
            (*Id the edges in each loop; each pair in ordered list a and append edge last->first*)
        LA = Map[SparseArray[#->Table[1,{Length[#]}],{Max[a],Max[a]}]&,Edges,{1}]; (*Create an AdjMat for each loop*)
        While[Dimensions[LA][[1]]>0,(*while loops*)
            c = Map[Total[BinSub[#,B],2]&,LA,{1}];(*calculate # of edges contributed to base by each loop*)
            (*drop loops that are not contributing*)
            LA = Delete[LA,Position[c,0]];
            a = Delete[a,Position[c,0]];
            c = Delete[c,Position[c,0]];
            If[ Total[c]>0,(*if contributing loops*)
                i = Position[c,Min[c]][[1]][[1]]; (*id first loop with minimum contribution*)
                B = BinSum[B,LA[[i]]];(*add edges of selected loop to base*)
                AppendTo[S,a[[i]]];(*add loop list to FBSILS*)
                LA = Drop[LA,{i}];(*drop loop AdjMat and edge list*)
                a = Drop[a,{i}];
            ];
        ]; (*end while loops*)
        S
    (* return loop list s=FBSILS *)
    ];



(* ::Subsubsection:: *)
(*Independent loop set*)


(* CEK: There may be an issue here whether one can properly distinguish edge lists and adjacency
lists.  Thus, I have chosen to make it a requirement that the function be given the edge list. It
is easy laster to allow for other input forms *)
FBIndependentLoopSet[el_?FBEdgeListQ] :=
    Block[ {debug,s,t,nodeLoop,nodeUsed,arcLoop,arcUsed,arcReserved,arcFree,loopList, arcList},
    (* 0. Initialize *)
        debug = False;
        arcUsed = {};
        nodeUsed = {1};
        arcReserved = Cases[el,{1,_}]; (* All arcs going from the first node reserved*)
        loopList = {};
        arcList = {};
        arcFree = Complement[el,arcReserved];
        If[ debug,
            Print["Start: arcReserved=",arcReserved]
        ];
        If[ debug,
            Print["       arcFree=",arcFree]
        ];
        While[arcReserved!={},
        (* 1. Pick the next reserved arc s->t*)
        {s,t} = arcReserved[[1]];
        arcReserved = Drop[arcReserved,1];
        AppendTo[arcUsed,{s,t}];
        If[ debug,
            Print["Step 1: s=",s,", t=",t]
        ];
        If[ debug,
            Print["        arcReserved=",arcReserved]
        ];
        If[ debug,
            Print["        arcUsed=",arcUsed]
        ];
        (* 2. Find a path back p=t->x1->x2->...->xn->s *)
        nodeLoop = ShortestPath[FromOrderedPairs[Join[arcUsed,arcFree],Type->Directed],t,s];
        arcLoop = Partition[nodeLoop,2,1];
        If[ debug,
            Print["Step 2: nodeLoop=",nodeLoop,", arcLoop=",arcLoop]
        ];
        (* 3. Add the loop s->t->x1->x2->...->xn->s to the loop list and s->t to the arc list*)
        AppendTo[loopList,FBCyclicSort[nodeLoop]];
        AppendTo[arcList,{s,t}];
        If[ debug,
            Print["Step 3: loopList=",loopList]
        ];
        (* 4. Add new nodes and arcs to the "green" set *)
        nodeUsed = Union[nodeUsed,nodeLoop];
        arcLoop = Complement[arcLoop,arcUsed];
        arcUsed = Join[arcUsed,arcLoop];
        arcFree = Complement[arcFree,arcLoop];
        If[ debug,
            Print["Step 4: nodeUsed=",nodeUsed]
        ];
        If[ debug,
            Print["        arcUsed=",arcUsed]
        ];
        If[ debug,
            Print["        arcFree=",arcFree]
        ];
        (* 5. Mark any blue arcs from the new points yellow *)
        arcReserved = Join[arcReserved,Flatten[Map[Cases[arcFree,{#,_}]&,nodeLoop],1]];
        arcFree = Complement[arcFree,arcReserved];
        If[ debug,
            Print["Step 5: arcReserved=",arcReserved]
        ];
        If[ debug,
            Print["        arcFree=",arcFree]
        ];
        If[ debug,
            Print["----------------------------------------------------------------------------------"]
        ];
        ];
        {loopList,arcList}
    ];


(* ::Section:: *)
(*LEEA Functions*)


(* ::Subsection:: *)
(*LEEA Info*)


SetAttributes[leeaInfo,HoldAll];
leeaInfo[mdlDefinitions_,msgList_] :=
    Block[ {var,lev,xv,yv,dsv,aux,n,p,nodelist,Mm,sc,el,edgelist,cl,clg,cm,cmg,clText,noesc,clr,cmr,loopsetText,loopnodes},
        Catch[
        Check[
        
        var = mdlDefinitions["Variables"];
        lev = mdlDefinitions["Levels"][[;;,1;;2]];
        xv = Map[First,lev];
        yv = Map[First,var];
        dsv = Map[Last,lev];
        aux = Map[Last,var];
        n = Length[xv];
        p = Length[yv];

        (* Nodes *)
        nodelist = Join[xv,yv];

        (* Adjacency matrix and strong components *)
        Mm = FBFromAdjacencyLists[
        Block[ {t = Join[dsv,aux]},
            Map[(Map[First,Position[t,#]])&,nodelist]
        ]
        ];
        sc = FBStrongComponents[Mm];

        (* Edges *)
        el = FBToEdgeList[Mm];
        edgelist = Map[nodelist[[#]]&,el,{2}];

        (* Directed cycles and cycle matrix *)
        cl = FBDirectedCycles[Mm];
        clg = FBGeodeticCycles[Mm];
        cm = FBCycleMatrix[cl,el];
        cmg = FBCycleMatrix[clg,el];
        
        (* No. of edges in strong components *)
        noesc = Length[Cases[Map[First[First[Position[sc,#]]]&,el,{2}],{x_,x_}]];

        (* A shorter independent loop set *)
        (* Shorter independent loop set *)
        clr = FBSILS[clg];

        (* Cycle matrix (reduced) *)
        cmr = FBCycleMatrix[clr,el];
        (*
        AppendTo[msgList,"Check: rank of Cm:       "<>ToString@FBRank[cm]];
        AppendTo[msgList,"Check: rank of Cmg:      "<>ToString@FBRank[cmg]];
        AppendTo[msgList,"*** Check that it is OK ***"];
        AppendTo[msgList,"Every loop in original cycle? "<>ToString@Union[Map[MemberQ[cl,#]&,clr]]];
        AppendTo[msgList,"Dimensions of cmr             "<>ToString@Dimensions[cmr]];
        AppendTo[msgList,"Rank of cmr                   "<>ToString@FBRank[cmr]];
        AppendTo[msgList,"Rank of cm                    "<>ToString@FBRank[cm]];
		*)
		
        (* Statistics and checks *)
        AssociationMap[Symbol,
                        {"mdlLEEA`Private`nodelist","mdlLEEA`Private`Mm","mdlLEEA`Private`sc","mdlLEEA`Private`el","mdlLEEA`Private`edgelist",
                        "mdlLEEA`Private`cl","mdlLEEA`Private`clg","mdlLEEA`Private`cm","mdlLEEA`Private`cmg","mdlLEEA`Private`noesc","mdlLEEA`Private`clr","mdlLEEA`Private`cmr"}
                        ]
        ,
        
        Throw[$Failed]
        ]
        ]
    ]



(* ::Subsection:: *)
(*Extract eigenvalues from real form Schur matrix *)


eigenvaluesFromRealSchur[s_List] := 
 Module[{dsub, dmain, dsuper, i, ev = {}},
  {dsub, dmain, dsuper} = Diagonal[s, #] & /@ {-1, 0, 1};
  Do[
   If[(! PossibleZeroQ@dsuper[[i]] || ! PossibleZeroQ@dsub[[i]]) && 
     dmain[[i + 1]] == dmain[[i]],
    AppendTo[ev, 
     Eigenvalues[{{dmain[[i]], dsuper[[i]]}, {dsub[[i]], 
        dmain[[i + 1]]}}]],
    AppendTo[ev, {dmain[[i]], dmain[[i + 1]]}];
    ];
   If[i == Length[dmain] - 2, AppendTo[ev, dmain[[i + 2]]]],
   {i, 1, Length[dsuper], 2}
   ];
  Flatten@ev
  ]
  



(* ::Subsection:: *)
(*LEEA Run*)


SetAttributes[leeaRun,HoldAll];
leeaRun[mdlDefinitions_,mdlData_,exprLEEAInfo_,msgList_] :=
    Block[ {SubNum,mdlSolveLEEA,xv,yv,dsv,aux,n,p,imp},
        Catch[
        Check[
        SubNum[n_] :=
            MapThread[Rule[ToExpression[#1],ToExpression[#2]]&,{StringReplace[imp[[1]]," "->""],imp[[n]]}];
        mdlSolveLEEA[] :=
            Block[ {##,nodelist,Mm,sc,el,edgelist,cl,clg,cm,cmg,clText,noesc,clr,cmr,loopsetText,loopnodes,eg,gain,lgr,egv,subEgToEgv,lg,Am,Bm,Cm,Dm,Um,Jm,edgeGain,loopGain,linkEla, loopEla,loopInf,outLoopEla,EV},

				(* extract work variables from data *)
                Set[Evaluate@Symbol[#] , mdlData[#]]&/@Keys@mdlData;
                Set[Evaluate@Symbol[#] , exprLEEAInfo[#]]&/@Keys@exprLEEAInfo;

                (* Directed cycles and cycle matrix *)
                clText = Map[Function[nodelist[[#1]]],clg,{2}];

                (* ILS list *)
                loopsetText = MapIndexed[{"Loop "<>ToString@First@#2,#1}&,SymbolName/@nodelist[[#]]&/@clr];
                (*Table[{"Loop "<>ToString[i],Map[nodelist[[#]]&,clr[[i]]]},{i,Length[clr]}];*)
                loopnodes = StringJoin@@Riffle[#,"\[Rule]"]&/@loopsetText[[;;,2]];
                (*Table[StringReplace[ToString[loopsetText[[i,2]]],{", "\[Rule]">","{"->"","}"->""}],{i,1,Length[loopsetText]}];*)

                (* Expressing loop and edge gains *)
                (* List edge gains g[i,j] and loop gains g[i] *)
                eg = Map[Apply[gain,#]&,el];

                (*List of edge gains gain[x,y]*)
                lgr = Table[gain[i],{i,Length[cmr]}];

                (*List of loop gains gain[i]*)
                (* Edge gains as functions of variables *)
                egv = Map[
                Apply[
                Function[{x1,x2},
                If[ x2<=n,
                    If[ x1<=n,
                        D[dsv[[x2]],xv[[x1]]],
                        D[dsv[[x2]],yv[[x1-n]]]
                    ],
                    If[ x1<=n,
                        D[aux[[x2-n]],xv[[x1]]],
                        D[aux[[x2-n]],yv[[x1-n]]]
                    ]
                ]
                ],
                #
                ]&,
                el
                ];

                (* Edge gains, as functions of variables *)
                subEgToEgv = Table[eg[[i]]->egv[[i]],{i,Length[eg]}];(* g[i,j]\[Rule]f(x,...)*)

                (* Loop gains *)
                lg = cmr.eg/.Plus->Times;

                (* Jacobian *)
                Am = Table[If[ MemberQ[edgelist,{xv[[j]],xv[[i]]}],
                               gain[j,i],
                               0
                           ],{i,n},{j,n}];
                Bm = Table[If[ MemberQ[edgelist,{yv[[j]],xv[[i]]}],
                               gain[j+n,i],
                               0
                           ],{i,n},{j,p}];
                Cm = Table[If[ MemberQ[edgelist,{xv[[j]],yv[[i]]}],
                               gain[j,i+n],
                               0
                           ],{i,p},{j,n}];
                Dm = Table[If[ MemberQ[edgelist,{yv[[j]],yv[[i]]}],
                               gain[j+n,i+n],
                               0
                           ],{i,p},{j,p}];
                Um = Inverse[IdentityMatrix[p]-Dm];
                Jm = Am+Bm.Um.Cm//Expand;

                (* Link/loop elasticities *)
                (* Although calculated, the current version of the procedure process does not export the link elasticities. *)

                (* Matrix method *)
                outLoopEla = {{"t","i","Re[z]","Im[z]","Loop nr","Loop","Nodes","gain","Re[e]","Im[e]","Abs[e]", "Re[I]", "Im[I]", "Abs[I]"}};
                Block[ {i,j,k,s,t,Ax,Bx,Cx,Dx,lx,rx,fx,gx,An,Bn,Cn,Dn,Un,Jn,cmrinv,z,ev,es,Rn,Ln,eLoopEla, linkElaMa, schur, schurEv,newEV,newRn,index},
                    cmrinv = N[PseudoInverse[Transpose[cmr]]];
                    linkElaMa = MapThread[If[z == 0, 0, (#1 #2 / z)] &,
	                    {Map[
		                    {Ax,Bx,Cx,Dx}[[#[[1]]]][#[[2]],#[[3]]] &,
		                    Map[Flatten[Position[{Am,Bm,Cm,Dm},#]]&,eg]
	                    ],
	                    eg
	                    }
                    ];

                    Ax[i_,j_] :=
                        lx[[i]]rx[[j]]An[[i,j]];
                    Bx[i_,j_] :=
                        lx[[i]]gx[[j]];
                    Cx[i_,j_] :=
                        fx[[i]]rx[[j]];
                    Dx[i_,j_] :=
                        fx[[i]]gx[[j]];
                    EV = outLoopEla = eLoopEla = {};
                    For[i = 2,i<=Length[imp],i++, (*for every t in the imported data set*)
	                    s = subEgToEgv /. SubNum[i];
	                    t = mdl`Time /. SubNum[i];
	                    An = Am /. s;
	                    Bn = Bm /. s;
	                    Cn = Cm /. s;
	                    Dn = Dm /. s;
	                    Un = Um /. s;
	                    Jn = Jm /. s;
	                    (*Print["\nJn : ", Jn];*)
	                    {ev,Rn} = Eigensystem[Jn];
	                    (*Print["Eigensystem : ", Eigensystem[Jn]];*)
	                    
	                    (*
	                    (* find Schur decomposition *)
	                    schur = SchurDecomposition[Jn, Pivoting->True];
	                    schurEv = eigenvaluesFromRealSchur[schur[[2]]];
	                    
	                    (* sort eigenvalues according to the diagonal entries of Schur matrix *)
	                    newEV = First@Flatten[FoldPairList[
	                    	{
                    		Nearest[#1, #2, 1],
                    		Delete[#1,Nearest[#1 -> "Index", #2, 1]]
                    		} &, 
	                   		ev, schurEv], {2}];
	                    index = Position[ev, #] & /@ newEV // Flatten;
	                    newRn = Rn[[index]];
	                    
	                    (*Print["Schur : ", schurEv]; 
	                    Print["Sorted : ",{newEV, newRn}];*)
	                    
	                    ev=newEV;
	                    Rn=newRn;
	                    *)
	                    
	                    Ln = Inverse[Transpose[Rn]] // Chop;
	                    edgeGain = eg /. s;
	                    loopGain = lg /. s;
	                    AppendTo[EV,{Re[ev],Im[ev]}];   (*EV has structure t,part,eigenvalue*)
	                    For[j = 1, j <= n, j++,                            (*for every eigenvalue*)
		                    z = ev[[j]];
		                    rx = Rn[[j]];
		                    lx = Ln[[j]];
		                    fx = lx.Bn.Un;
		                    gx = Un.Cn.rx;
		                    If[ z != -1000,
		                        linkEla = linkElaMa /. s;
		                        loopEla = cmrinv.linkEla;
		                        loopInf = loopEla * ev[[j]];
		                        AppendTo[eLoopEla,{loopGain,loopEla,loopInf}]
		                    (*eLoopEla has strcutre eigenvalue,values,loop*)
		                	]
	                    ];
	                    AppendTo[outLoopEla,eLoopEla]; (*outLoopEla has strcture t,z,v,loop*)
	                    eLoopEla = {};
                    ];
                ];
                AssociationMap[Symbol,
                                {"mdlLEEA`Private`loopsetText","mdlLEEA`Private`loopnodes","mdlLEEA`Private`edgeGain",
                                "mdlLEEA`Private`loopGain","mdlLEEA`Private`EV","mdlLEEA`Private`outLoopEla","mdlLEEA`Private`linkEla",
                                "mdlLEEA`Private`loopEla","mdlLEEA`Private`loopInf",
                                "mdlLEEA`Private`subEgToEgv","mdlLEEA`Private`Am","mdlLEEA`Private`Bm","mdlLEEA`Private`Cm","mdlLEEA`Private`Dm","mdlLEEA`Private`Um","mdlLEEA`Private`Jm"}
                                ]
            ] & @@ (Join@@Lookup[mdlDefinitions,{"Levels","Variables"}][[;;,;;,1]]);
        
        mdlSolveLEEA[],
        
        Throw[$Failed]
        ]
        ]
    ]


(* ::Subsection:: *)
(*Eigenvalues Output*)


SetAttributes[leeaEigenvaluesOut,HoldAll];
leeaEigenvaluesOut[mdlDat_,leeaDat_,size_,export_,controls_,display_,exportcontent_,timepos_,reset_] :=
    Pane[
	    DynamicModule[{
	    				i,times,t,fullt,EV,n,evp,v,EVT,EVP,\[Lambda],legendColumns,colors,colorsFull,
	    				
	    				type,lePlot,leTable,
	    				
	    				initializeControls,initializeDisplay,emptyDisplay,emptyOutput,waitIndicator,
	    				
	    				display1,display2,export11,export12,export2,finished1,finished2,redraw1,redraw2,tabtype,
	    				notebookBackground,tabsize,plotWidth = size[[1]]-40,plotHeight = size[[2]]-100},
	    	
	    	initializeControls[resetType_]:=Module[{},
        		If[resetType == 0, 
	        		controls = ConstantArray[{},5];
        			type = 1;
        			tabtype = 1; 
        			timepos = -1;
        			lePlot = emptyOutput;
        			leTable = emptyOutput;
   			];
                If[resetType == 1 || resetType == 0,
                	\[Lambda] = Range[mdlDat["mdlLEEA`Private`n"]];
                	v = 1;
					controls[[2]] = type;
	                controls[[3]] = \[Lambda];
	                controls[[4]] = v;
                ];
                If[resetType == 2 || resetType == 0,
        			t = 1;
        			controls[[5]] = t;
                ];
                ];

			(* initialize variables *)
		    notebookBackground=CurrentValue[EvaluationNotebook[], Background];
		    If[notebookBackground===None,notebookBackground=White];

            emptyOutput = {};
            emptyDisplay = ConstantArray[emptyOutput,2];
            initializeDisplay = ConstantArray[emptyDisplay,2];
            
			waitIndicator = {
				Graphics[{Opacity[0.7], notebookBackground, Rectangle[Scaled[{0, 0}], Scaled[{1, 1}]]}, PlotRange -> {{0, plotWidth}, {0, plotHeight}},ImageSize -> {plotWidth,plotHeight}],
				Pane[
					Row[{
						ProgressIndicator[Appearance -> "Indeterminate", ImageSize -> {170,30}],
						Style[" Updating output ...", 12, Italic, FontFamily -> "Helvetica"]
					}],
					{plotWidth,plotHeight},
					Alignment->Center
				]
			};

		    EV = leeaDat["mdlLEEA`Private`EV"];
		    n = mdlDat["mdlLEEA`Private`n"];
		    
           	colorsFull = Cases[ListPlot[Table[{i},{i,mdlDat["mdlLEEA`Private`n"]}]],RGBColor[x__],Infinity];
            
			times = Table[i-1->mdlDat["mdlLEEA`Private`imp"][[i]][[1]],{i,2,Length[mdlDat["mdlLEEA`Private`imp"]]}];
            fullt = Table[mdlDat["mdlLEEA`Private`imp"][[i]][[1]],{i,2,Length[mdlDat["mdlLEEA`Private`imp"]]}];
			
		    (* if controls were initiliazed, restore the layout, otherwise initialize *)
			If[ controls =!= {},
				tabtype = controls[[1]];
				type = controls[[2]];
                \[Lambda] = controls[[3]];
                v = controls[[4]];
				t = controls[[5]];
				t = Min[t,Length[times]];
				,
             	initializeControls[0];
            ];

            (* if display is not empty, restore it, otherwise initialize *)
			If[display =!= "" && display =!= initializeDisplay, 
	            display1 = display[[1,type]];
	            lePlot = display[[1,1]];
	            leTable = display[[1,2]];

	            display2 = display[[2]];
	            
				redraw1 = redraw2 = False;
				,
				display = initializeDisplay;
				finished1 = finished2 = False;  
				redraw1 = redraw2 = True;
				display1 = display2 = emptyDisplay;
			];
            
			If[exportcontent =!= "" && exportcontent =!= initializeDisplay, 
	            export11 = exportcontent[[1,1]];
	            export12 = exportcontent[[1,2]];
	            
	            export2 = exportcontent[[2]];
				,
				exportcontent = initializeDisplay;
			];
            
            
		    tabsize = {size[[1]]-30,size[[2]]-65};
		    
			Dynamic[
	        	If[reset == 1, initializeControls[tabtype],
	        		If[reset == 2, initializeControls[0]]
	    		];
				reset = 0;

				controls[[1]] = tabtype;

				export["FileName"] = 
					If[tabtype == 1,
						"Eigenvalues Evolution",
						"Eigenvalues Plot"
					];

				If[tabtype == 2 || type == 1,
	                export["ButtonText"] = "Export as PDF";
	                export["Type"] = "PDF";
	                ,
                    export["ButtonText"] = "Export as CSV";
                    export["Type"] = "CSV";
				];
				
				export["Content"] = 
					Switch[tabtype,
						1,If[type == 1,export11, export12],
						2,export2
					];
				
				Switch[tabtype,
					2, timepos = t,
					_, timepos = -1
				];
				
			    TabView[{
			    	{1, Style["\[Lambda] Evolution",Italic] -> Module[{},
						Pane[
					    	Column[
						    	{
							    Style[Row[
							    	{
							    	"   ", SetterBar[Dynamic@type, {1 -> "Plot", 2 -> "Table"}],
		            			    "\tPart ", SetterBar[Dynamic@v, { 1-> "Re", 2 -> "Im"}],
			            			"\tEigenvalue ", Dynamic[TogglerBar[Dynamic[\[Lambda]], Range[mdlDat["mdlLEEA`Private`n"]]]]
		            				}
		            				],
		            				FontFamily->"Helvetica"
	            				],
				
								DynamicWrapper[
									Dynamic[
										Overlay[
											{
											If[display1 =!= emptyDisplay && display1 =!= emptyOutput,display1, ""],
											If[finished1 === False, Sequence@@waitIndicator, ""]
											},
											All	
										],
										TrackedSymbols:>{finished1,display1}
									],
				            		
									timepos = -1;							

					            	If[mdlDat =!= "" && leeaDat =!= "",
										controls[[2]] = type;
						                controls[[3]] = \[Lambda];
						                controls[[4]] = v;
					            	];
									
									If[(mdlDat =!= "" && leeaDat =!= "") 
										&& (
											(redraw1 
											&& !(
												(type == 1 && display1 === leTable) 
												|| (type == 2 && display1 === lePlot)
												)
											) 
											|| display1 === emptyDisplay || display1 === emptyOutput),
										
							           	finished1 = False;
							           	
							            EVT = Map[Prepend[leeaDat["mdlLEEA`Private`EV"][[#, v, Sort[\[Lambda]]]],times[[#]][[2]]]&,Range[Length[times]]];
							            EVP = leeaDat["mdlLEEA`Private`EV"][[All,v,Sort[\[Lambda]]]];
							            legendColumns = Ceiling[Length[\[Lambda]] / 20];
						                colors = colorsFull[[Sort[\[Lambda]]]];
					
						                lePlot = 
						                	Column[{
							                        Module[ {table},
							                            table[pairs_] :=
							                                Multicolumn[TableForm[#,TableDirections->Row] & /@ pairs,legendColumns];
							                                
							                            export11 = exportcontent[[1,1]] = ListLinePlot[Transpose[{fullt, #}] & /@ Transpose[EVP],
							                                (*DataRange->{Min[t],Max[t]},*)
							                                PlotStyle->colors,
							                                AxesLabel->{"Time","Eigenvalue"},
							                                PlotLegends->Placed[
							                                    LineLegend[Sort[\[Lambda]],
							                                        LegendLabel->"Eigenvalue",
							                                        LegendLayout->table,
							                                        LegendMargins->0,
							                                        LabelStyle->{Italic,11,FontFamily->"Helvetica"}
							                                        ],
							                                    Right
							                                    ],                                                                  
							                            PlotRange->All,
							                            AspectRatio->(plotHeight-50)/(plotWidth-80),
							                            ImageSize->{plotWidth-80,plotHeight-50},
							                            (*ImagePadding->{{All,0},{All,0}},*)
							                            AxesLabel->{Style["t",Italic],""}
							                            ]
							                        ],
							                        Style["Eigenvalue numbers are assigned in order of decreasing absolute value; this sometimes results in disrupted eigenvalue trajectories.\nPlease take this into account when assessing the role of a behavior mode at a particular point in time",Small,Italic]
						                	}];
						                	
				                        export12 = exportcontent[[1,2]] = Prepend[EVT,{"Time"}~Join~Sort[\[Lambda]]];

						                leTable = 
						                	Column[{
							                        Style[TableForm[Map[PaddedForm[#,{7,5}]& ,EVT[[;;,2;;]],{2}],TableHeadings->{fullt,Sort[\[Lambda]]},TableAlignments->Center],FontFamily->"Helvetica"],
								                    Style["Eigenvalue numbers are assigned in order of decreasing absolute value; this sometimes results in disrupted eigenvalue trajectories.\nPlease take this into account when assessing the role of a behavior mode at a particular point in time",Small,Italic]
					                		}];

										finished1 = True
					            	];
					            	redraw1 = True;

									If[display =!= "",
										If[lePlot =!= emptyOutput && leTable =!= emptyOutput, 
											display1 = If[type == 1, lePlot, leTable]; 
											display[[1,1]] = lePlot; 
											display[[1,2]] = leTable;
										]
									];
					            	export["Content"] = If[type == 1, export11, export12];
									,
									TrackedSymbols:>{type,v,\[Lambda]},
									SynchronousUpdating->False
								]
					    		}
				    		],
							tabsize,
							
				    		Scrollbars->{True,True},
				    		ImageSizeAction->"Clip",
			    			AppearanceElements->None
						]
	    			]
	    			},

			    	{2, Style["\[Lambda] Plot @ time",Italic] -> Module[{},
						Pane[
					    	Column[
						    	{
							    Style[Row[{"    Time ", Slider[Dynamic[t],{1,Length[times],1}], " ", Dynamic@NumberForm[times[[t,2]],{100,1}]}],FontFamily->"Helvetica"],
				
								DynamicWrapper[
									Dynamic[
										Overlay[
											{
											If[display2 =!= emptyDisplay && display2 =!= emptyOutput,display2, ""],
											If[finished2 === False, Sequence@@waitIndicator, ""]
											},
											All	
										],
										TrackedSymbols:>{finished2,display2}
									],
								            		
					          		Switch[tabtype,
										2, timepos = t,
										_, timepos = -1
									];

					            	If[mdlDat =!= "" && leeaDat =!= "",
										controls[[5]] = t;
					            	];
									
									If[(mdlDat =!= "" && leeaDat =!= "") && (redraw2 || display2 === emptyDisplay || display2 === emptyOutput),
	
							           	finished2 = False;
									    
									    evp = {EV[[t, 1, #]], EV[[t, 2, #]]} & /@ Range[n];
					
				 					    display2 = display[[2]] = export2 = exportcontent[[2]] = 
				 					    	ListPlot[MapThread[Callout[#1,#2,LeaderSize->{20,Automatic,10}] &,{evp}~Join~{StringForm["EV ``",#] & /@ Range@Length@evp}], 
											    AxesOrigin->{0,0},
											    AxesLabel->{"Re","Im"},
											    PlotLabel->"Time = "<>ToString@times[[t,2]],
											    PlotMarkers->{Automatic,Large},
											    PlotRange->{
											    	{Min[0, 1.05*Min[leeaDat["mdlLEEA`Private`EV"][[All,1]]]], Max[0, 1.05*Max[leeaDat["mdlLEEA`Private`EV"][[All,1]]]]},
											     	{Min[0, 1.05*Min[leeaDat["mdlLEEA`Private`EV"][[All,2]]]], Max[0, 1.05*Max[leeaDat["mdlLEEA`Private`EV"][[All,2]]]]}
										    	},
											    PlotRangePadding->{{Scaled[0.05`],Scaled[0.05`]},{Scaled[0.05`],Scaled[0.05`]}},
											    GridLines->Automatic,
											    GridLinesStyle->Directive[LightGray, Dashed],
											    AspectRatio->plotHeight/plotWidth,
											    ImageSize->{plotWidth,plotHeight}
										    (*ImagePadding->{{All,All},{All,0}}*)
										    ];
									    
										finished2 = True
						    		];
									redraw2 = True;
									
									export["Content"] = export2;
									,
									TrackedSymbols:>{t},
									SynchronousUpdating->False
								]
					    		}
				    		],
							tabsize,
							
				    		Scrollbars->{False,False},
				    		ImageSizeAction->"Clip",
			    			AppearanceElements->None
						]
	    			]
	    			}

	    			},
		    		Dynamic@tabtype
			    ],
	        
			    TrackedSymbols:>{tabtype,type,reset},
			    SynchronousUpdating->True
    		]
    		
    		
	    ],
	    size-10,
	    FrameMargins->{{0,0},{0,10}},
	    Scrollbars->{False,False},
	    ImageSizeAction->"Clip",
	    AppearanceElements->None
    ]



SetAttributes[leeaLoopsOut,HoldAll];
leeaLoopsOut[mdlDat_,leeaDat_,size_,export_,controls_,display_,exportcontent_,timepos_,reset_] :=
    Pane[
	    DynamicModule[{
	    				i,times,fullt,\[Lambda],lgLegendColumns,liLegendColumns,colors,colorsFull,waitIndicator,
	    				
	    				loopsetText,loopnodes,loopTable,headersText,outLoopEla,
	    				
	    				loops,LGP,LGM,lgt,lgPlot,lgTable,lgtype,
	    				
	    				liColors,LIP,LIPN,liLoops,
	    				
	    				EV,littype,litLambda,litLoops,litt,til,LIT,litLIP,loopsOrdering,zoom,allLoopsAbs,allLoopsRe,litPlot,litTable,
	    				
	    				initializeControls,initializeDisplay,emptyDisplay,emptyOutput,display1,display2,display3,display4,export1,export21,export22,export3,export41,export42,finished1,finished2,finished3,finished4,redraw1,redraw2,redraw3,redraw4,tabtype,
	    				tabsize,notebookBackground,plotWidth = size[[1]]-40,plotHeight = size[[2]]-100},
	    	
	    	(* define local functions *)
	    	initializeControls[resetType_]:=Module[{},
        		If[resetType == 0, 
	        		controls = ConstantArray[{},12];
        			lgtype = 1;
        			littype = 1;
        			tabtype = 1;
        			timepos = -1;
        			zoom = 1;
        			lgPlot = emptyOutput;
        			lgTable = emptyOutput;
        			litPlot = emptyOutput;
        			litTable = emptyOutput;
    			];
                If[resetType == 2 || resetType == 0,
                	lgt = Range[Length[times]];
            		loops = Range[Length[loopnodes]];
					controls[[2]] = lgtype;
	                controls[[3]] = lgt;
	                controls[[4]] = loops;
                ];
                If[resetType == 3 || resetType == 0,
                	\[Lambda] = 1;
            		liLoops = Range[Length[loopnodes]];
	                controls[[5]] = \[Lambda];
	                controls[[6]] = liLoops;
                ];
                If[resetType == 4 || resetType == 0,
                	litLambda = 1;
                	litt = 1;
            		litLoops = Range[Length[loopnodes]];
            		til = Min[Length[loopnodes], 10];
					controls[[7]] = littype;
	                controls[[8]] = litLambda;
	                controls[[9]] = litLoops;
	                controls[[10]] = litt;
	                controls[[11]] = til;
                ];
                ];
                
            emptyOutput = {};
            emptyDisplay = ConstantArray[emptyOutput,2];
            initializeDisplay = ConstantArray[emptyDisplay,4];
            
			waitIndicator = {
				Graphics[{Opacity[0.7], notebookBackground, Rectangle[Scaled[{0, 0}], Scaled[{1, 1}]]}, PlotRange -> {{0, plotWidth}, {0, plotHeight}},ImageSize -> {plotWidth,plotHeight}],
				Pane[
					Row[{
						ProgressIndicator[Appearance -> "Indeterminate", ImageSize -> {170,30}],
						Style[" Updating output ...", 12, Italic, FontFamily -> "Helvetica"]
					}],
					{plotWidth,plotHeight},
					Alignment->Center
				]
			};

			(* initialize variables *)
		    notebookBackground=CurrentValue[EvaluationNotebook[], Background];
		    If[notebookBackground===None,notebookBackground=White];

		    headersText = {"#","Length","Participating Nodes"};
		    loopsetText = leeaDat["mdlLEEA`Private`loopsetText"];
		    loopnodes = leeaDat["mdlLEEA`Private`loopnodes"];
		    loopTable = Table[{i,Length[loopsetText[[i,2]]],loopnodes[[i]]},{i,1,Length[loopnodes]}];
		    
			times = Table[i-1->mdlDat["mdlLEEA`Private`imp"][[i]][[1]],{i,2,Length[mdlDat["mdlLEEA`Private`imp"]]}];
            fullt = Table[mdlDat["mdlLEEA`Private`imp"][[i]][[1]],{i,2,Length[mdlDat["mdlLEEA`Private`imp"]]}];
            
		    outLoopEla = leeaDat["mdlLEEA`Private`outLoopEla"];
		    colorsFull = Cases[ListPlot[Table[{i},{i,Length[loopnodes]}]],RGBColor[x__],Infinity];

		    EV = leeaDat["mdlLEEA`Private`EV"];
		    
		    (* if controls were initiliazed, restore the layout, otherwise initialize *)
			If[ controls =!= {},
				tabtype = controls[[1]];
				lgtype = controls[[2]];
                lgt = controls[[3]];
                loops = controls[[4]];
			    \[Lambda] = controls[[5]];
				liLoops = controls[[6]];
				littype = controls[[7]];
                litLambda = controls[[8]];
                litLoops = controls[[9]];
                litt = controls[[10]];
				litt = Min[litt,Length[times]];
                til = controls[[11]];
				,
             	initializeControls[0];
            ];
            
            (* if display is not empty, restore it, otherwise initialize *)
			If[display =!= "" && display =!= initializeDisplay, 
	            display1 = display[[1]];
	            
	            display2 = display[[2,lgtype]];
	            lgPlot = display[[2,1]];
	            lgTable = display[[2,2]];
	            
	            display3 = display[[3]];

	            display4 = display[[4,littype]];
	            litPlot = display[[4,1]];
	            litTable = display[[4,2]];
	            
				redraw1 = redraw2 = redraw3 = redraw4 = False;
				,
				display = initializeDisplay;
				finished1 = finished2 = finished3 = finished4 = False;  
				redraw1 = redraw2 = redraw3 = redraw4 = True;
				display1 = display2 = display3 = display4 = emptyDisplay;
			];

			If[exportcontent =!= "" && exportcontent =!= initializeDisplay, 
	            export1 = exportcontent[[1]];
	            
	            export21 = exportcontent[[2,1]];
	            export22 = exportcontent[[2,2]];
	            
	            export3 = exportcontent[[3]];

	            export41 = exportcontent[[4,1]];
	            export42 = exportcontent[[4,2]];
				,
				exportcontent = initializeDisplay;
			];

            
		    tabsize = {size[[1]]-30,size[[2]]-65};
		    
			Dynamic[
	        	If[reset == 1, initializeControls[tabtype],
	        		If[reset == 2, initializeControls[0]]
	    		];
				reset = 0;
				
				controls[[1]] = tabtype;

				export["FileName"] = 
					Switch[tabtype,
						1,"Loop List",
						2,"Loop Gains",
						3,"Loop Influence on EV " <> ToString@\[Lambda],
						4,"Loop Influence on EV " <> ToString@litLambda <> " at Time " <> ToString@times[[litt,2]]
					];
					
				If[(tabtype == 2 && lgtype == 1) || (tabtype == 4 && littype == 1) || tabtype == 3,
	                export["ButtonText"] = "Export as PDF";
	                export["Type"] = "PDF";
	                ,
                    export["ButtonText"] = "Export as CSV";
                    export["Type"] = "CSV";
				];

				export["Content"] = 
					Switch[tabtype,
						1,export1,
						2,If[lgtype == 1,export21, export22],
						3,export3,
						4,If[littype == 1,export41, export42]
					];
					
          		Switch[tabtype,
					4, timepos = litt,
					_, timepos = -1
				];

			    TabView[{
			    	{1, Style["Loop List",Italic] -> Module[{},
						Pane[
					    	Column[
						    	{
								DynamicWrapper[
									Dynamic[
										Overlay[
											{
											If[display1 =!= emptyDisplay && display1 =!= emptyOutput,display1, ""],
											If[finished1 === False, Sequence@@waitIndicator, ""]
											},
											All,
											1	
										],
										TrackedSymbols:>{finished1,display1}
									],
				            		
									If[(mdlDat =!= "" && leeaDat =!= "") && (redraw1 || display1 === emptyDisplay || display1 === emptyOutput),
										
							           	finished1 = False;
							           	
									    export1 = exportcontent[[1]] = Prepend[MapAt[StringReplace[#,"\[Rule]"->"->"]&,loopTable,{All,3}],headersText];
					
						                display1 = display[[1]] = Pane[
										    Column[{
										    	Style[TableForm[loopTable,TableHeadings->{None,Style[#,Bold]&/@headersText},TableAlignments->Left], FontFamily->"Helvetica"]
										    }],
										    {size[[1]]-30,size[[2]]-65},
										    FrameMargins->{{10,0},{0,10}},
										    Scrollbars->True,
										    ImageSizeAction->"Clip",
										    AppearanceElements->None
									    ];
									    
										finished1 = True
					            	];
					            	
					            	redraw1 = True;
					            	
					            	export["Content"] = export1;
									,
									TrackedSymbols:>{},
									SynchronousUpdating->False
								]
					    		}
				    		],
							tabsize,
							
				    		Scrollbars->{False,False},
				    		ImageSizeAction->"Clip",
			    			AppearanceElements->None
						]
	    			]
	    			},

			    	{2, Style["Loop Gains",Italic] -> Module[{},
						Pane[
					    	Column[
						    	{
							    Style[
								    Dynamic[If[ lgtype == 1,
									    Row[{
									        "   ",SetterBar[Dynamic@lgtype,{1->"Plot",2->"Table"}],
								            "\tLoops ",Dynamic@TogglerBar[Dynamic[loops],Range[Length[loopnodes]]]
								    	}],
						                Column[{
										    Row[{
										        "   ",SetterBar[Dynamic@lgtype,{1->"Plot",2->"Table"}],
									            "\tLoops ",Dynamic@TogglerBar[Dynamic[loops],Range[Length[loopnodes]]]
						                	}],
						                    Row[{"   Time ",Dynamic[TogglerBar[Dynamic[lgt],NumberForm[#,{100,1}] & /@ Association@times // Normal,Appearance->"Horizontal"]]}]
						                }]
						            ],
						            TrackedSymbols :> {lgtype}
								    ], 
		            				FontFamily->"Helvetica"
	            				],
				
								DynamicWrapper[
									Dynamic[
										Overlay[
											{
											If[display2 =!= emptyDisplay && display2 =!= emptyOutput,display2, ""],
											If[finished2 === False, Sequence@@waitIndicator, ""]
											},
											All	
										],
										TrackedSymbols:>{finished2,display2}
									],
								            		
					            	If[mdlDat =!= "" && leeaDat =!= "",
									    controls[[2]] = lgtype;
									    controls[[3]] = lgt;
									    controls[[4]] = loops;
					            	];
					          		
									If[(mdlDat =!= "" && leeaDat =!= "") 
										&& (
											(redraw2 
											&& !(
												(lgtype == 1 && display2 === lgTable) 
												|| (lgtype == 2 && display2 === lgPlot)
												)
											) 
											|| display2 === emptyDisplay || display2 === emptyOutput   
											),
										
										finished2 = False;
					
									    lgLegendColumns = Ceiling[Length[loops]/20];
										
										lgt = Cases[lgt, a_Integer /; a <= Length@times];
						
							            LGP = Flatten[outLoopEla[[All,{1},{1},Sort[loops]]],2];(*first 1 selects the first z, second 1 grabs the gain*)
							            colors = colorsFull[[Sort[loops]]];
							             
							            Module[ {table},
							            	table[pairs_] :=
							                	Multicolumn[TableForm[#,TableDirections->Row]&/@pairs,lgLegendColumns];
											lgPlot = export21 = exportcontent[[2,1]] = ListLinePlot[Transpose[{times[[;;,2]], #}] & /@ Transpose[LGP],
							                 	(*DataRange->{Min[times[[All,2]]],Max[times[[All,2]]]},*)
							                	PlotStyle->colors,
							                    PlotRange->All,
							                    PlotLegends->Placed[
							                    	LineLegend[
							                        	Sort[loops],
							                            LegendLabel->"Loop",
							                            LegendLayout->table,
							                            LegendMargins->0,
							                            LabelStyle->{Italic,11,FontFamily->"Helvetica"}
							                        ],
							                    Right
							                    ],
							                AspectRatio->(plotHeight-20)/(plotWidth-60),
							                ImageSize->{plotWidth-60,plotHeight-20},
							                (*ImagePadding->{{All,0},{All,10}},*)
							                AxesLabel->{"Time","Gain"}
							                ]
							            ];
									             
							            LGM = Flatten[outLoopEla[[lgt,{1},{1},loops]],2]; (*first 1 selects the first z, second 1 grabs the gain*)
							            
							            (* add time column *)
							            export22 = MapThread[Prepend[#1,#2] &,{N[LGM[[All,Ordering[LGM[[1]]]]]],times[[Sort@lgt,2]]}];
						             	
						             	(* add headers *)
										PrependTo[export22,{"Time"}~Join~loops[[Ordering[LGM[[1]]]]]];
										
										exportcontent[[2,2]] = export22;
										
							            lgTable = Style[
											TableForm[Map[PaddedForm[#,{7,5}]&, N[LGM[[All,Ordering[LGM[[1]]]]]],{2}], 
								            	TableHeadings->{times[[Sort@lgt,2]],loops[[Ordering[LGM[[1]]]]]},
								             	TableAlignments->Center
						             		],
						             		FontFamily->"Helvetica"
						             	];
										finished2 = True
						    		];
									redraw2 = True;
									
									If[display =!= "",
										If[lgPlot =!= emptyOutput && lgTable =!= emptyOutput, 
											display2 = If[lgtype == 1, lgPlot, lgTable]; 
											display[[2,1]] = lgPlot; 
											display[[2,2]] = lgTable;
										]
									];
					            	export["Content"] = If[lgtype == 1, export21, export22];
									,
									TrackedSymbols:>{lgtype,lgt,loops},
									SynchronousUpdating->False
								]
					    		}
				    		],
							tabsize,
							
				    		Scrollbars->{True,True},
				    		ImageSizeAction->"Clip",
			    			AppearanceElements->None
						]
	    			]
	    			},
	    			
			    	{3, Style["Loop Influence",Italic] -> Module[{},
						Pane[
					    	Column[
						    	{
							    Style[
								    Column[{
								        Row[{"   Eigenvalue ",Dynamic[SetterBar[Dynamic[\[Lambda]],Range[mdlDat["mdlLEEA`Private`n"]]]]}],
								        Row[{"   Loops ",Dynamic[TogglerBar[Dynamic[liLoops],Range[Length[loopnodes]]]]}]
								    }],
		            				FontFamily->"Helvetica"
	            				],
				
								DynamicWrapper[
									Dynamic[
										Overlay[
											{
											If[display3 =!= emptyDisplay && display3 =!= emptyOutput,display3, ""],
											If[finished3 === False, Sequence@@waitIndicator, ""]
											},
											All	
										],
										TrackedSymbols:>{finished3,display3}
									],
								            		
					            	If[mdlDat =!= "" && leeaDat =!= "",
									    controls[[5]] = \[Lambda];
			    						controls[[6]] = liLoops;
					            	];

									If[(mdlDat =!= "" && leeaDat =!= "") && (redraw3 || display3 === emptyDisplay || display3 === emptyOutput),
										
										finished3 = False;

									    liLegendColumns = Ceiling[Length[liLoops]/20];
									    
									    LIP = Flatten[outLoopEla[[All,{\[Lambda]},{3},Sort[liLoops]]],2];
									    (*extract the influence metric for all times, and the selected eigenvalue and loops;
									    lambda can only take a single value, the {3} grabs the loop influence*)
						
									    LIPN = Sign[Re[LIP]]*Abs[LIP];
									    (*Calculate the signed influence*)
									    
									    liColors = colorsFull[[Sort[liLoops]]];
										
									    display3 = display[[3]] = 
										    Module[ {table},
										        table[pairs_] :=
										            Multicolumn[TableForm[#,TableDirections->Row]&/@pairs,liLegendColumns];
										        export3 = exportcontent[[3]] = ListLinePlot[Transpose[{times[[;;,2]], #}] & /@ Transpose[LIPN],
											        DataRange->{Min[times[[All,2]]],Max[times[[All,2]]]},
											        PlotStyle->liColors,
											        PlotLegends->Placed[
											            LineLegend[
											                Sort[liLoops],
											                LegendLabel->"Loop",
											                LegendLayout->table,
											                LegendMargins->0,
											                LabelStyle->{Italic,11,FontFamily->"Helvetica"}
											            ],
											            Right
										        		],
											        PlotRange->All,
											        AspectRatio->(plotHeight-40)/(plotWidth-60),
								                 	ImageSize->{plotWidth-60,plotHeight-40},
											        ImagePadding->{{All,0},{All,10}},
											        AxesLabel->{Style["t",Italic],Style["Sign[Inf]",Italic]},
											        PlotLabel -> "Eigenvalue " <> ToString@\[Lambda]
											        ]					        
						                ];
										finished3 = True
						    		];
									redraw3 = True;
									
					            	export["Content"] = export3;
									,
									TrackedSymbols:>{\[Lambda],liLoops},
									SynchronousUpdating->False
								]
					    		}
				    		],
							tabsize,
							
				    		Scrollbars->{True,True},
				    		ImageSizeAction->"Clip",
			    			AppearanceElements->None
						]
	    			]
	    			},

			    	{4, Style["Loop Influence @ time",Italic] -> Module[{},
						Pane[
					    	Column[
						    	{
							    Style[
								    Column[{
									    Row[{
									        "   ",SetterBar[Dynamic@littype,{1->"Plot",2->"Table"}],
									        "   Time ", Slider[Dynamic[litt],{1,Length[times],1}]," ", Dynamic@If[litt <= Length[times],NumberForm[times[[litt,2]],{100,1}],NumberForm[times[[-1,2]],{100,1}]],
									        "   Top Loops ", Dynamic[SetterBar[Dynamic[til],Range[Min[Length[loopnodes], Length[litLoops], 10]]]]
									    }],
									    Row[{"   Eigenvalue ", Dynamic[SetterBar[Dynamic[litLambda],Range[mdlDat["mdlLEEA`Private`n"]]]]}],
										Row[{"   Loops ", Dynamic@TogglerBar[Dynamic[litLoops],Range[Length[loopnodes]]]}]
								    }],
		            				FontFamily->"Helvetica"
	            				],
				
								DynamicWrapper[
									Dynamic[
										Overlay[
											{
											If[display4 =!= emptyDisplay && display4 =!= emptyOutput,display4, ""],
											If[finished4 === False, Sequence@@waitIndicator, ""]
											},
											All,
											1
										],
										TrackedSymbols:>{finished4,display4}
									],
								            		
					          		Switch[tabtype,
										4, timepos = litt,
										_, timepos = -1
									];

									til = Min[til,Length[litLoops]];
									litt = Min[litt,Length[times]];

					            	If[mdlDat =!= "" && leeaDat =!= "",
										controls[[7]] = littype;
						                controls[[8]] = litLambda;
						                controls[[9]] = litLoops;
						                controls[[10]] = litt;
						                controls[[11]] = til;
					            	];
					            	
									If[(mdlDat =!= "" && leeaDat =!= "") 
										&& (
											(redraw4 
											&& !(
												(littype == 1 && display4 === litTable) 
												|| (littype == 2 && display4 === litPlot)
												)
											) 
											|| display4 === emptyDisplay || display4 === emptyOutput
										),
					            	
										finished4 = False;

									    liLegendColumns = Ceiling[Length[litLoops]/20];
									    
									    litLIP = Flatten[outLoopEla[[All,{litLambda},{3},Sort[liLoops]]],2];
									    (*extract the influence metric for all times, and the selected eigenvalue and loops;
									    lambda can only take a single value, the {3} grabs the loop influence*)
						
									   	LIT = Flatten[outLoopEla[[{litt}, {litLambda}, {3}, litLoops]]];(*lambda can only take a single value, the 3 grabs the loop influence*)
									   	litLIP = Map[Join[{Abs[LIT[[#]]], Re[LIT[[#]]]}] &, Range[Length[litLoops]]];
									   	loopsOrdering = Ordering[Abs[LIT], til, Greater];
								    	allLoopsAbs = Abs[Flatten[outLoopEla[[;;, {litLambda}, {3}, litLoops]]]];
								    	allLoopsRe = Re[Flatten[outLoopEla[[;;, {litLambda}, {3}, litLoops]]]];
										
									    litPlot =  
										    Module[{plot},
										    	plot[plotrange_]:=
										    	ListPlot[MapThread[Callout[#1,#2,LeaderSize->{20,Automatic,10}] &,{litLIP[[loopsOrdering]]}~Join~{StringForm["Loop ``",#] & /@ litLoops[[loopsOrdering]]}],
											   		AxesOrigin -> {0, 0},
											   		AxesLabel -> {"Abs[I]", "Re[I]"}, 
											   		PlotLabel->"Time = " <> ToString@times[[litt,2]] <> ", Eigenvalue " <> ToString@litLambda <> " = " <> ToString@TraditionalForm[EV[[litt,1,litLambda]] + EV[[litt,2,litLambda]] I],
											    	PlotMarkers -> {Automatic, Large}, 
											    	PlotRange->
											    		If[plotrange === 1,
											    			{
												    		{0, 1.05*Max[litLIP[[;;]][[All,1]]]},
												     		{Min[0, 1.05*Min[litLIP[[;;]][[All,2]]]], Max[0, 1.05*Max[litLIP[[;;]][[All,2]]]]}
											    			},
												     		{
												    		{0, 1.05*Max[allLoopsAbs]},
												     		{Min[0, 1.05*Min[allLoopsRe]], Max[0, 1.05*Max[allLoopsRe]]}
												     		}
											    		],
											    	PlotRangeClipping -> False,
											    	GridLines->Automatic,
												    GridLinesStyle->Directive[LightGray, Dashed],
									                AspectRatio->(plotHeight-70)/(plotWidth),
								                 	ImageSize->{plotWidth,plotHeight-70}
										    	];
										    	
										    	Column[{
										    		DynamicWrapper[
										    			FlipView[{plot[0],plot[1]},Dynamic[zoom]]
										    			,
										    			export41 = exportcontent[[4,1]] = If[zoom === 1, plot[0], plot[1]];
										    			,
										    			TrackedSymbols :> {zoom}
										    		],
													Style["Click on the graph to zoom in to current loops",8,Italic]
										    	}]
										    ];
										   
									    export42 = MapThread[Prepend[#1,#2] &,{{N[Abs[LIT[[loopsOrdering]]]], N[Re[LIT[[loopsOrdering]]]], N[Im[LIT[[loopsOrdering]]]]},{"Abs", "Re", "Im"}}];
								        (* add headers *)
								        PrependTo[export42,{""} ~Join~ litLoops[[loopsOrdering]]];
								        
								        exportcontent[[4,2]] = export42;
								        
						             	litTable = Style[
									    	TableForm[Map[PaddedForm[#,{7,5}]&,{N[Abs[LIT[[loopsOrdering]]]], N[Re[LIT[[loopsOrdering]]]], N[Im[LIT[[loopsOrdering]]]]},{2}],
									     		TableHeadings -> {{"Abs", " Re", " Im"}, Map[ToString, litLoops[[loopsOrdering]]]}, 
									     		TableSpacing -> {1, 1},
							             		TableAlignments->Center
						             		],
							             	FontFamily->"Helvetica"
					             	 	];
					             	 					     
										finished4 = True
						    		];
									redraw4 = True;
									
									If[display =!= "",
										If[litPlot =!= emptyOutput && litTable =!= emptyOutput, 
											display4 = If[littype == 1, litPlot, litTable]; 
											display[[4,1]] = litPlot; 
											display[[4,2]] = litTable;
										]
									];
					            	export["Content"] = If[littype == 1, export41, export42];
									,
									TrackedSymbols:>{littype,litLambda,litLoops,litt,til},
									SynchronousUpdating->False
								]
					    		}
				    		],
							tabsize,
							
				    		Scrollbars->{False,False},
				    		ImageSizeAction->"Clip",
			    			AppearanceElements->None
						]
	    			]
	    			}

	    			},
		    		Dynamic@tabtype
			    ],
	        
			    TrackedSymbols:>{tabtype,lgtype,littype,\[Lambda],litLambda,litt,reset},
			    SynchronousUpdating->True
    		]
    		
    		
	    ],
	    size-10,
	    FrameMargins->{{0,0},{0,10}},
	    Scrollbars->{False,False},
	    ImageSizeAction->"Clip",
	    AppearanceElements->None
    ]


(* ::Section:: *)
(*DDWA Functions*)


(* ::Subsection:: *)
(*DDWA Run*)


SetAttributes[ddwaRun,HoldAll];
ddwaRun[mdlDefinitions_,mdlData_,exprLEEAInfo_,leeaDat_,ddwaTimeVectorPos_,msgList_] :=
    Block[ {SubNum,ComplexToWeight,mdlSolveDDWA,xv,yv,dsv,aux,n,p,imp,Theta,levInit,behaviorModes,behaviorModesAll,idx,c,levNum,
    		evProjections,evProjectionsWeights,evProjectionsWeightsNormalized,evProjectionsConstants,evProjectionsConstantsNormalized,cNormalized},
    	
        Catch[
        Check[
        SubNum[nt_] :=
            MapThread[Rule[ToExpression[#1],ToExpression[#2]]&,{StringReplace[imp[[1]]," "->""],imp[[nt]]}];
            
       	ComplexToWeight[comp_,levInitt_]:=

			Module[{j,\[Theta],Thetat={},W={},idxt={},ct}, 
			                               
				ct=Map[Last[levInitt[[#]]]&,Range[n]]; (*itialize constant with initial state values*)      
				                                       
				For[j = 1,j <= Dimensions[comp][[1]],j++,                (*For every mode*)
					If[j < Dimensions[comp][[1]] && Table[0,{n}] == Chop[comp[[j]]-Conjugate[comp[[j+1]]]],     
						{                                                                                                            (*if part of conjugate*)
							W = Append[W, 2*Sqrt[Re[comp[[j]]]^2+Im[comp[[j]]]^2]];                            
							                                                                                                           (*recorde the combined weight *)
							(*the next two instructions are equivalent to the theta presented in the appendix of the paper*)
							\[Theta] = Arg[comp[[j]]]+Pi/2;
							Map[If[Im[comp[[j,#]]] >= 0 && Re[comp[[j,#]]] < 0,\[Theta][[#]] = \[Theta][[#]]-2Pi] &, Range[Length[\[Theta]]]];
	
							(*\[Theta] = ArcTan[Re[comp[[j]]]/-Im[comp[[j]]]];
							Map[If[-Im[comp[[j,#]]] < 0 ,\[Theta][[#]] = \[Theta][[#]] + Pi] &, Range[Length[\[Theta]]]];*)
							
							Thetat = Append[Thetat,\[Theta]];                 
							ct = ct-2*Re[comp[[j]]];
							idxt = Append[idxt,j];                                                      (*record in idx extracted eigenvalue*)
							j++;                                                                        (*skip next mode -- is the conjugate *)
							Thetat = Append[Thetat,{}];                                            		(*but add a black theta for it*)
						},
						{                                                                               (*else, ie if not conjugate *)
							W = Append[W,Re[comp[[j]]]];                                       			(*record the real part *)
							Thetat = Append[Thetat,{}];
							ct = ct-Re[comp[[j]]];                                                                                                   
							idxt = Append[idxt,j];                                                      (*record in idx extracted eigenvalue*) 
						}
					];
				];
				(*c=c/.levNum//.auxa/.par/.datNum;*)
				{W,Thetat,ct,idxt}
			];
            
        mdlSolveDDWA[] :=
            Block[ {##,nodelist,Mm,sc,el,edgelist,cl,clg,cm,cmg,clText,noesc,clr,cmr,loopsetText,loopnodes,eg,gain,lgr,egv,subEgToEgv, SubNumRules,
            		lg,Am,Bm,Cm,Dm,Um,Jm,edgeGain,loopGain,linkEla, loopEla,loopInf,outLoopEla,EV,i,Tau,
            		ddwaTimePos,var,lev,dat,par,modelpar,datNum,auxa,heldParm,heldAux,heldLev,
            		d,db,J,nrt0,L1,R1,Lb,z,nz,A1,lin,comp,WBase,y,yFormatted,yFormattedPar,stockEquations,stockEquationsPar,stockEquationsEVTerms,stockEquationsWeights,stockFullEVProjectionsBase,stockFullEVProjections,
            		m,M1,r,k,PElas,PInfER,PInfEI,PElasER,PElasEI},


				(* extract work variables from data *)
                Set[Evaluate@Symbol[#] , mdlData[#]]&/@Keys@mdlData;
                Set[Evaluate@Symbol[#] , exprLEEAInfo[#]]&/@Keys@exprLEEAInfo;
                Set[Evaluate@Symbol[#] , leeaDat[#]]&/@Keys@leeaDat;

				ddwaTimePos = ddwaTimeVectorPos+1;
				SubNumRules = SubNum[ddwaTimePos];

		        var = mdlDefinitions["Variables"];
		        lev = mdlDefinitions["Levels"][[;;,1;;2]];
		        dat = mdlDefinitions["Data"];
		        par = #[[1]]->#[[2]]&/@mdlDefinitions["Constants"];
		        modelpar = #[[1]]->#[[2]]&/@mdlDefinitions["Parameters"];
		        
		        
				(* set initial levels from imp structure *)
				levInit = MapThread[#1 -> #2 &,{mdlDefinitions["Levels"][[;;,1]],imp[[ddwaTimePos,Flatten[Position[imp[[1]],ToString@#] & /@ (mdlDefinitions["Levels"][[;;,1]]),Infinity]]]}];
				
		        (*levInit = #[[1]]->#[[3]]&/@mdlDefinitions["Levels"];*)
		        
		        (*idx = Position[imp[[1]], ToString@#] & /@ lev[[;;,1]] // Flatten;
		        levNum = MapThread[Rule[#1, #2] &, {lev[[All, 1]], imp[[ddwaTimePos, idx]]}];
		        idx = Position[imp[[1]], ToString@#] & /@ dat[[;;,1]] // Flatten;
				datNum = MapThread[Rule[#1, #2] &, {dat[[;;,1]], imp[[ddwaTimePos,idx]]}];*)
				
				(* Computation of the driving vector *)
				(* 
				This section recognizes that the model might be driven by a constant set of factors.  Since stocks cannot be directly influenced by these factors (other than through rates) the new system notation is 
					x_dot=Ax+By
					y=Cx+Dy+Eu
				where u is the vector of system parameters and E links the effects of parameters to auxiliary variables.  Since we are not interested in changing  E, we can change the second equation to
					y=Cx+Dy+d.
				 Solving for d
					d=(I-D)y-Cx
				Now we can use the equation
					x_dot=J.x+B.(I-D)^-1.d
				to estimate the initial net change rates, where J is still the systems Jacobian
					J=A+B.(I-D)^-1.C
				*)
				
				
				
				(* Computation of weight matrix *)
				(* 
				After computing Alpha, the routine separates the zero and non-zero eigenvalues and estimates the appropriate wieght 
				(\[Alpha]r for zero eigenvalues and \[Alpha]r/\[Lambda] for non-zero values) and adds them up to create the equations. 
				*)                   
				          
				d = Dot[IdentityMatrix[p] - Dm, yv] - Dot[Cm, Map[Last, levInit]];
				
				Block[{##,mdl`Time},
					(* set data variables from the imp structure *)
					mdl`Time = imp[[ddwaTimePos,1]];
					
					MapThread[Set,{mdlDefinitions["Data"][[;;,1]],imp[[ddwaTimePos,Flatten[Position[imp[[1]],ToString@#] & /@ (mdlDefinitions["Data"][[;;,1]]),Infinity]]]}];

					MapThread[Set,{mdlDefinitions["Levels"][[;;,1]],levInit[[;;,2]]}];
	
					(* set parameters *)
					MapThread[Set,{par[[;;,1]],par[[;;,2]]}];
					
					(* set variables *)
					MapThread[Set,{mdlDefinitions["Variables"][[;;,1]],mdlDefinitions["Variables"][[;;,2]]}];
	
					db = d //. subEgToEgv; 
					J = Jm //. subEgToEgv;        
					nrt0 = J.xv + Bm.Um.db //. subEgToEgv;
	
					{L1, R1} = Eigensystem[J];           					(*L->e-values; R->e-vectors*)
					(*Print@Chop@{L1, R1};*)
					Lb = L1;
					z = Count[Chop[L1], 0];             				    (*find the # of evalues=0*) 
					nz = Dimensions[L1][[1]] - z ;    						(*find the # of evalues\[NotEqual]0*)

					A1 = nrt0.Inverse[R1];		 							(*Alpha*)
					lin = Tau*Take[A1, -z]*Take[R1, -z];	 				(*Take works because 0 are at end of evalue vector *)
					comp = (Take[A1, nz]/Take[L1, nz])*Take[R1, nz];  		(*comp is (mode,stock) array*) 
					
					(*Print@comp;*)
					
					{WBase,Theta,c,idx} = ComplexToWeight[comp,levInit];							(*complex weights into std. weights*)
				];
				
				(*Construct Mode Vector*)
				y = Exp[Tau*Re[L1]];		                            (*Exp [real part*t]*)
				yFormatted = TraditionalForm /@ Exp[Re[L1]*Tau];
				stockEquationsEVTerms = TraditionalForm /@ Exp[Re[L1[[idx]]]*"\[Tau]"];
				stockEquationsWeights = ConstantArray[1.,{n,n}];
				
				Map[
					If[Im[L1[[#]]] != 0,
						y[[#]] = y[[#]]*Sin[Im[L1[[#]]]*Tau + Theta[[#]]];  		(*If Im add Sine & Theta*)
						yFormatted[[#]] = yFormatted[[#]] * (TraditionalForm /@ Sin[TraditionalForm /@ (Im[L1[[#]]]*Tau + Theta[[#]])]);
						stockEquationsWeights[[#]] = (TraditionalForm /@ Sin[(NumberForm[#,{7,2}] & /* TraditionalForm) /@ (Im[L1[[#]]]*Tau + Theta[[#]])]);
						,
				   		y[[#]] = Table[y[[#]], {n}];							       	(*else add copies of real part *)
				   		yFormatted[[#]] = Table[yFormatted[[#]], {n}];
				   		(*stockEquationsWeights[[#]] = 1;*)
				   	] &, 
				  Range[n]
				];
				
				stockEquationsWeights = Transpose[Chop[WBase]*stockEquationsWeights[[idx]]];
				
				behaviorModesAll = Switch[Lb[[#]],
						_Complex, If[# < Length[Lb] && Re[Lb[[#]]] == Re[Lb[[#+1]]] && Abs@Im[Lb[[#]]] == Abs@Im[Lb[[#+1]]],ToString@# <> "," <> ToString[#+1], Nothing],
						_, ToString@#
					] & /@ idx;
				
				behaviorModes = Table[behaviorModesAll,	Length@Lb];
				behaviorModes = Delete[behaviorModes, Position[WBase\[Transpose], 0.]];			
				
				evProjections = Transpose[y[[idx]]];
				
				evProjectionsWeights = Transpose[WBase];
				cNormalized = If[Chop[Abs[c[[#]]]] == 0, 1, Abs[c[[#]]]] & /@ Range@Length[c] ;
				evProjectionsWeightsNormalized = (evProjectionsWeights[[#]] / cNormalized[[#]]) & /@ Range@Length[evProjectionsWeights];

				evProjectionsConstants = c+Total[lin];
				evProjectionsConstantsNormalized = evProjectionsConstants / cNormalized; 
				
				(* screen output form *)
				stockEquations = Thread[Append[Chop[WBase]*yFormatted[[idx]],Chop[Total[lin]]]]; (* append the linear part if exists, adding up by eigenvectors and threading through stocks *)
				stockEquations = MapThread[Prepend,{stockEquations, Chop[c]}]; (* prepend the constant *)
				
				(* list of all full EV projections including constant, base case *)
				stockFullEVProjectionsBase = Thread[Append[Chop[WBase]*y[[idx]],Chop[Total[lin]]]]; (* append the linear part if exists, adding up by eigenvectors and threading through stocks *)
				stockFullEVProjectionsBase = MapThread[Prepend,{stockFullEVProjectionsBase, Chop[c]}]; (* prepend the constant *)

				(* plot form *)
				m = Transpose[Chop[WBase]*y[[idx]]];							(*Generate products *)
				Map[(m[[#]] = Total[m[[#]]]) &, Range[n]];             			(*Add products*)
				m = m + Total[lin];                                     		(*lin Totaled in case of more than 1 evalue=0*)
				M1 = Table[First[lev[[i]]] -> m[[i]] + c[[i]], {i,n}];  		(*Add Stock name and constant*)
				r = FullSimplify[Table[M1[[i]][[2]] / If[Chop[Abs[c[[i]]]]==0,1,Abs[c[[i]]]], {i, Length[M1]}]];

				(* Estimate DDW Elasticities to Parameters *)
				PElas=PInfER=PInfEI=PElasER=PElasEI={};
				
				stockFullEVProjections = Table[{{{},{}},{}},Length[par]];
				stockEquationsPar = Table[{},Length[par]];
				
				For[k = 1, k <= Length[par], k++,  								(*for every edge *)

					Block[{##,mdl`Time,T1,dm,J1,nrt02,L2,R2,z2,nz2,A2,lin2,comp2,W2,Theta2,c2,idx2,y2,parm,auxm,j},
						
						stockFullEVProjections[[k,1,1]] = par[[k,1]];
						
						(* set data variables from the imp structure *)
						mdl`Time = imp[[ddwaTimePos,1]];
					
						MapThread[Set,{mdlDefinitions["Data"][[;;,1]],imp[[ddwaTimePos,Flatten[Position[imp[[1]],ToString@#] & /@ (mdlDefinitions["Data"][[;;,1]]),Infinity]]]}];
						MapThread[Set,{mdlDefinitions["Levels"][[;;,1]],levInit[[;;,2]]}];
						
						(* perturb one constant and set all constants *)
						T1 = ConstantArray[1,Length[par]];
						T1[[k]] = 1.001;
						T1 *= par[[;;,2]];
						MapThread[Set,{par[[;;,1]],T1}];
						
						stockFullEVProjections[[k,1,2]] = par[[k,1]];

						(* set variables *)
						MapThread[Set,{mdlDefinitions["Variables"][[;;,1]],mdlDefinitions["Variables"][[;;,2]]}];

						dm = d //.subEgToEgv;
						J1 = Jm //.subEgToEgv;      
						nrt02 = J1.xv+Bm.Um.dm //.subEgToEgv;

						{L2,R2}=Eigensystem[J1];  
						(*Print@Chop@{L2,R2};*) 
						z2 = Count[Chop[L2], 0];             				    (*find the # of evalues=0*) 
						nz2 = Dimensions[L2][[1]] - z2 ;    					(*find the # of evalues\[NotEqual]0*)
						
						A2 = nrt02.Inverse[R2]; 								(*Alpha*)
						lin2 = Tau*Take[A2, -z2]*Take[R2, -z2];	 				(*Take works because 0 are at end of evalue vector *)

																				(*takes the same value of nz extimated in the base case*)
						comp2=(Take[A2,nz2]/Take[L2,nz2])*Take[R2,nz2];           (*comp is (mode,stock) array*)
						{W2,Theta2,c2,idx2} = ComplexToWeight[comp2,levInit];   (*complex weights into std. weights*)

						(*Construct Mode Vector*)
						y2 = Exp[Tau*Re[L2]];		                            (*Exp [real part*t]*)
						yFormattedPar = TraditionalForm /@ Exp[Re[L2]*Tau];
						Map[
							If[Im[L2[[#]]] != 0,
								y2[[#]] = y2[[#]]*Sin[Im[L2[[#]]]*Tau + Theta2[[#]]];  		(*If Im add Sine & Theta*)
								yFormattedPar[[#]] = yFormattedPar[[#]] * (TraditionalForm /@ Sin[TraditionalForm /@ (Im[L2[[#]]]*Tau + Theta2[[#]])]);
								,
						   		y2[[#]] = Table[y2[[#]], {n}];
						   		yFormattedPar[[#]] = Table[yFormattedPar[[#]], {n}];							       	(*else add copies of real part *)
						   	] &, 
						  Range[n]
						];

						(* list of all full EV projections including constant *)
						stockFullEVProjections[[k,2]] = Thread[Append[Chop[W2]*y2[[idx2]],Chop[Total[lin2]]]]; (* append the linear part if exists, adding up by eigenvectors and threading through stocks *)
						stockFullEVProjections[[k,2]] = MapThread[Prepend,{stockFullEVProjections[[k,2]], Chop[c2]}]; (* prepend the constant *)

						(* screen output form *)
						stockEquationsPar[[k]] = Thread[Append[Chop[W2]*yFormattedPar[[idx2]],Chop[Total[lin2]]]]; (* append the linear part if exists, adding up by eigenvectors and threading through stocks *)
						stockEquationsPar[[k]] = MapThread[Prepend,{stockEquationsPar[[k]], Chop[c2]}]; (* prepend the constant *)


						Quiet[PElas=Append[PElas,Chop[1000*((W2-WBase)/WBase)]]];             					(*PElas is (par, mode, state) array*)
						Quiet[PInfER=Append[PInfER,Chop[1000*(Re[L2[[idx]]]-Re[Lb[[idx]]])]]]; 	(*PElas ER&EI are (par,mode) arrays*)
						Quiet[PInfEI=Append[PInfEI,Chop[1000*(Im[L2[[idx]]]-Im[Lb[[idx]]])]]];
						Quiet[PElasER=Append[PElasER,Chop[1000*(Re[L2[[idx]]]-Re[Lb[[idx]]])/Re[Lb[[idx]]]]]]; 	(*PElas ER&EI are (par,mode) arrays*)
						Quiet[PElasEI=Append[PElasEI,Chop[1000*(Im[L2[[idx]]]-Im[Lb[[idx]]])/Im[Lb[[idx]]]]]];
					];
				];
				
(*				Print@y;
				Print@idx;
				Print@stockFullEVProjectionsBase;
				Print@par[[;;,1]];
				Print@stockFullEVProjections;
*)				
				(*Print[PElas,PElasER,PElasEI];*)

                AssociationMap[Symbol,
                                {
	                            	"mdlLEEA`Private`r","mdlLEEA`Private`idx","mdlLEEA`Private`behaviorModesAll","mdlLEEA`Private`behaviorModes",
	                            	"mdlLEEA`Private`evProjections","mdlLEEA`Private`evProjectionsWeights","mdlLEEA`Private`evProjectionsWeightsNormalized",
	                            	"mdlLEEA`Private`evProjectionsConstants","mdlLEEA`Private`evProjectionsConstantsNormalized",
	                            	"mdlLEEA`Private`M1","mdlLEEA`Private`Lb","mdlLEEA`Private`stockEquations", "mdlLEEA`Private`stockEquationsPar", "mdlLEEA`Private`stockEquationsEVTerms","mdlLEEA`Private`stockEquationsWeights",
	                            	"mdlLEEA`Private`WBase","mdlLEEA`Private`PElas","mdlLEEA`Private`PInfER","mdlLEEA`Private`PInfEI","mdlLEEA`Private`PElasER","mdlLEEA`Private`PElasEI",
	                            	"mdlLEEA`Private`ddwaTimePos"
                            	}
                            ]
            ] & @@ (Join@@Lookup[mdlDefinitions,{"Constants","Data","Levels","Variables"}][[;;,;;,1]]);
        
        mdlSolveDDWA[],
        
        Throw[$Failed]
        ]
        ]
    ]




(* ::Subsection:: *)
(*DDWA Stock DD*)


SetAttributes[ddwaDDPlot,HoldAll];
ddwaDDPlot[mdlDefinitions_,mdlDat_,ddwaDat_,size_,export_,controls_,timepos_,reset_] :=
    Pane[
	    DynamicModule[{type,stocks,lev,M1,idx,behaviorModes,ddwaTimePos,s,times,tm,r,tmax,i,
	    	stockIndex,stockEquations,stockEquationsPar,stockEquationsTable,stockEquationsEVTerms,stockEquationsWeights,initializeControls,legendList,legendColumns,
	    	plotList,plotListNormalized,
	    	evProjections,evProjectionsWeights,evProjectionsWeightsNormalized,evProjectionsConstants,evProjectionsConstantsNormalized,
	    	content,plotWidth = size[[1]]-170,plotHeight = size[[2]]-150, tabsize},
	    	
	    	initializeControls[resetType_]:=Module[{},
	    			If[resetType == 0, type = 1];
	        		controls = ConstantArray[{},3];
	            	stockIndex = 1;
	            	tm = tmax;
	                ];

		    r = ddwaDat["mdlLEEA`Private`r"];
		    idx = ddwaDat["mdlLEEA`Private`idx"];
		    behaviorModes = ddwaDat["mdlLEEA`Private`behaviorModes"];
		    M1 = ddwaDat["mdlLEEA`Private`M1"];
		    ddwaTimePos = ddwaDat["mdlLEEA`Private`ddwaTimePos"];
		    stockEquations = ddwaDat["mdlLEEA`Private`stockEquations"];
		    stockEquationsPar = ddwaDat["mdlLEEA`Private`stockEquationsPar"];
		    stockEquationsEVTerms = ddwaDat["mdlLEEA`Private`stockEquationsEVTerms"];
		    stockEquationsWeights = ddwaDat["mdlLEEA`Private`stockEquationsWeights"];
		    
			evProjections = ddwaDat["mdlLEEA`Private`evProjections"];
			
			evProjectionsWeights = ddwaDat["mdlLEEA`Private`evProjectionsWeights"];
			evProjectionsWeightsNormalized = ddwaDat["mdlLEEA`Private`evProjectionsWeightsNormalized"];
			
			evProjectionsConstants = ddwaDat["mdlLEEA`Private`evProjectionsConstants"];
			evProjectionsConstantsNormalized = ddwaDat["mdlLEEA`Private`evProjectionsConstantsNormalized"];
		    
		    lev = mdlDefinitions["Levels"][[;;,1;;2]];
			stocks = Table[i ->  StringDelete[ToString@(mdlDefinitions["Levels"][[i,1]]),"mdl`"], {i, 1, Length[mdlDefinitions["Levels"]]}];
	    	times = Table[i-1 -> mdlDat["mdlLEEA`Private`imp"][[i]][[1]],{i,ddwaTimePos,Length[mdlDat["mdlLEEA`Private`imp"]]}];
	    	tmax = First@Cases[mdlDefinitions["Parameters"],{mdl`FINALTIME,x_}:>x];
	    	
	    	stockEquationsTable = Join[
	    			{Join[{Style["Constant",Bold]}, Style[NumberForm[#,{7,2}],Bold] & /@ (stockEquationsEVTerms //. {mdlLEEA`Private`Tau -> "\[Tau]"}),{Style["\[Tau]",Bold]}]},
	    			MapThread[Join[{#1[[1]]}, #2, {#1[[-1]]}] &,{stockEquations,stockEquationsWeights //. {mdlLEEA`Private`Tau -> "\[Tau]"}}]
	    	];
	    	stockEquationsTable = MapThread[Join[{#1}, #2] &,{Style[#, Bold] & /@ Join[{"Stock"}, stocks[[;;,2]]], stockEquationsTable}];
	    	
	    	
		    export["Type"] = "PDF";
		    export["ButtonText"] = "Export as PDF";
		    
		    content = Table[{},3];
		    
		    tabsize = {size[[1]]-35,plotHeight+85};
		    
		    Dynamic[
	        	If[reset == 1, initializeControls[1],
	        		If[reset == 2, initializeControls[0]]
	    		];
            	reset = 0;
            	
				controls[[1]] = type;

				export["Content"] = content[[type]];
				(*Print@export["Content"];*)
				
				export["FileName"] = 
					If[type == 2,
						"Stock Absolute Dynamic Decomposition Plot",
						If[type == 3,
							"Stock Normalized Dynamic Decomposition Plot",
							"Stock Decomposition Equations"
						]
				];
				
				timepos = -1;

			    TabView[{
			    	{1, "Equations" -> Module[{},
						Pane[
							(*Dynamic[
								export["Content"] = content[[type]];*)
								content[[1]] = 
									Column[{
										Item[
											Style["Stock equations",Bold],
											Alignment->Left
										],
										PaddedForm[
											GridBox[stockEquationsTable,
												ColumnAlignments->{Left,Center},
												GridBoxDividers->{"Rows"->{False,True}}
											] // DisplayForm,
											{7,2}
										]
										},
										Dividers->{None,{2->Green}}
									]
									(*,
									TrackedSymbols :> {type}
							]*),
								
							tabsize,
							
				    		Scrollbars->{True,True},
				    		ImageSizeAction->"Clip",
			    			AppearanceElements->None
						]
	    			]
	    			},
			    	{2, "Absolute" -> Pane[
					    Module[ {table},
					        table[pairs_] :=
					            Multicolumn[TableForm[#,TableDirections->Row]&/@pairs,legendColumns];
						    Manipulate[
						    	export["Content"] = content[[type]];
						    	
								controls[[2]] = stockIndex;
								controls[[3]] = tm;
								
								s = lev[[stockIndex]][[1]] /. M1 // Chop;
								
								plotList = DeleteCases[
									Prepend[(evProjections[[stockIndex]]) * (evProjectionsWeights[[stockIndex]]),
										If[Chop[evProjectionsConstants[[stockIndex]]] != 0.,evProjectionsConstants[[stockIndex]],Nothing]
									],
									0.
								];
				
								legendList = If[Chop[evProjectionsConstants[[stockIndex]]] != 0.,Join[{"Constant"}, #], #] & @ (Style[Subscript["\[Lambda]",#], Italic] & /@ behaviorModes[[stockIndex]]); 
								
								(*NumberForm[#,{5,5}] & /@ Cases[plotList,a_:>Exponent[a,E] / mdlLEEA`Private`Tau];*) (*Table[PaddedForm[Exponent[r[[stockIndex,i]], E]/mdlLEEA`Private`Tau, {5, 4}], {i,Length[r[[stockIndex]]]}];*)
								legendColumns = Ceiling[Length[legendList]/20];

							    content[[2]] = 
							    	Column[{
								    	(*Style[StringForm["`1` = `2`",stocks[[stockIndex,2]], NumberForm[s //. {mdlLEEA`Private`Tau -> "\[Tau]"}, {7, 5}]],Italic],*)
								    	(*Print@Style[StringForm["`1` = `2`",stocks[[stockIndex,2]], NumberForm[(Defer @* Plus) @@ DeleteCases[(stockEquations[[stockIndex]] //. {mdlLEEA`Private`Tau -> "\[Tau]"}), 0. | 0], {7, 5}]],Italic];
								    	Print@Style[StringForm["`1` = `2`",stocks[[stockIndex,2]], NumberForm[(Defer @* Plus) @@@ DeleteCases[(stockEquationsPar[[;;,stockIndex]] //. {mdlLEEA`Private`Tau -> "\[Tau]"}), 0. | 0], {7, 5}]],Italic];*)
								    	
								    	Show[{
										    Plot[s,
												{mdlLEEA`Private`Tau, 0, tm}, 
												PlotStyle->{Thickness[0.01],Orange},
												AxesOrigin -> {0,0},
												PlotRange -> All, 
												AxesLabel -> {"Time since \!\(\*SubscriptBox[\(\[Tau]\), \(0\)]\)", stocks[[stockIndex,2]]}, (*PlotStyle->{Dashed,Dotted, DotDashed, Thick, Thin},*)
												PlotLabel -> "\!\(\*SubscriptBox[\(\[Tau]\), \(0\)]\) = " <> ToString[mdlDat["mdlLEEA`Private`imp"][[ddwaTimePos,1]]],
								                AspectRatio->plotHeight/plotWidth,
									            ImageSize->{plotWidth,plotHeight}
												   
											 ],
											 Plot[Evaluate[plotList],
											 	{mdlLEEA`Private`Tau, 0, tm},
											 	AxesOrigin -> {0,0},
											 	PlotRange -> All,
											 	PlotLegends ->
													LineLegend[
												   		legendList,
												   		LegendLabel->"Component",
										                LegendLayout->table,
										                LegendMargins->0,
										                LabelStyle->{Italic,11,FontFamily->"Helvetica"}
											   		],
											   		PlotStyle -> Dashed
										   		]
								    	}]
							    		}
							    		(*Dividers->{None,{2->Green}}*)
							    	],
							    	
							    	Row[{
									    "    Stocks ",
									    PopupMenu[Dynamic@stockIndex,stocks,FieldSize->Small],
									    "\tMaximum time since \!\(\*SubscriptBox[\(\[Tau]\), \(0\)]\) ",
									    Slider[Dynamic@tm,{Min[1,tmax/10],tmax}],
									    " ",
									    Dynamic@tm
								    	},
								    	Alignment->Center
								    ],
								    
								    ControlPlacement->Top,
								    TrackedSymbols:>{stockIndex,tm,reset},
								    SynchronousUpdating->True,
								    Paneled->False,
								    FrameMargins->{{0,0},{0,0}},
								    AppearanceElements->None
						    ]
					    ],
					    
			    		tabsize,
			    		
			    		Scrollbars->{True,False},
			    		ImageSizeAction->"Clip",
		    			AppearanceElements->None
	    			]
	    			}, 
		    		{3, "Normalized" -> Pane[
					    Module[ {table},
					        table[pairs_] :=
					            Multicolumn[TableForm[#,TableDirections->Row]&/@pairs,legendColumns];
				          	
				          	Manipulate[
				          		export["Content"] = content[[type]];
				          		
								controls[[2]] = stockIndex;
								controls[[3]] = tm;
								
								plotListNormalized = DeleteCases[
										Prepend[(evProjections[[stockIndex]]) * (evProjectionsWeightsNormalized[[stockIndex]]),
										If[Chop[evProjectionsConstantsNormalized[[stockIndex]]] != 0.,evProjectionsConstantsNormalized[[stockIndex]],Nothing]
									],
									0.
								];
									
								legendList = If[Chop[evProjectionsConstants[[stockIndex]]] != 0.,Join[{"Constant"}, #], #] & @ (Style[Subscript["\[Lambda]",#], Italic] & /@ behaviorModes[[stockIndex]]); 
								
								(*NumberForm[#,{5,5}] & /@ Cases[plotList,a_:>Exponent[a,E] / mdlLEEA`Private`Tau];*) (*Table[PaddedForm[Exponent[r[[stockIndex,i]], E]/mdlLEEA`Private`Tau, {5, 4}], {i,Length[r[[stockIndex]]]}];*)
								legendColumns = Ceiling[Length[legendList]/20];

							    content[[3]] = Plot[Evaluate[plotListNormalized],
									{mdlLEEA`Private`Tau, 0, tm},
									AxesOrigin -> {0,0}, 
									PlotRange -> All, 
									AxesLabel -> {"Time since \!\(\*SubscriptBox[\(\[Tau]\), \(0\)]\)", stocks[[stockIndex,2]]}, (*PlotStyle->{Dashed,Dotted, DotDashed, Thick, Thin},*)
									PlotLabel -> "\!\(\*SubscriptBox[\(\[Tau]\), \(0\)]\) = " <> ToString[mdlDat["mdlLEEA`Private`imp"][[ddwaTimePos,1]]],
									PlotLegends ->
										LineLegend[
									   		legendList,
									   		LegendLabel->"Component",
							                LegendLayout->table,
							                LegendMargins->0,
							                LabelStyle->{Italic,11,FontFamily->"Helvetica"}
								   		],
					                AspectRatio->plotHeight/plotWidth,
						            ImageSize->{plotWidth,plotHeight}
								 ],
							    	Row[{
									    "    Stocks ",
									    PopupMenu[Dynamic@stockIndex,stocks,FieldSize->Small],
									    "\tMaximum time since \!\(\*SubscriptBox[\(\[Tau]\), \(0\)]\) ",
									    Slider[Dynamic@tm,{Min[1,tmax/10],tmax}],
									    " ",
									    Dynamic@tm
								    	},
								    	Alignment->Center
								    ],
								    
								    ControlPlacement->Top,
								    TrackedSymbols:>{stockIndex,tm,reset},
								    SynchronousUpdating->True,
								    Paneled->False,
								    FrameMargins->{{0,0},{0,0}},
								    AppearanceElements->None
						    ]
								 
					    ],
					    
			    		tabsize,
			    		Scrollbars->{True,False},
			    		ImageSizeAction->"Clip",
		    			AppearanceElements->None
	    			]
	    			}},
		    		Dynamic@type
			    ],
	        
			    (*ControlPlacement->Top,*)
			    TrackedSymbols:>{type,reset},
			    SynchronousUpdating->True,
			    (*Paneled->False,
			    AppearanceElements->None,*)
			    Initialization :> (
					If[ controls=!={},
						type = controls[[1]];
						stockIndex = controls[[2]];
						tm = controls[[3]]
						,
	                 	initializeControls[0];
	                ]
                )
    		]
	    ],
	    size-10,
	    FrameMargins->{{0,0},{0,10}},
	    ImageSizeAction->"Clip",
	    AppearanceElements->None
    ]



(* ::Subsection:: *)
(*DDWA Elasticities*)


SetAttributes[ddwaElasticity,HoldAll];
ddwaElasticity[mdlDefinitions_,mdlDat_,ddwaDat_,size_,export_,controls_,timepos_,reset_] :=
    Pane[
	    DynamicModule[{type,stocks,lev,par,M1,Lb,WBase,PElas,PInfER,PInfEI,PElasER,PElasEI,idx,behaviorModesAll,ddwaTimePos,
	    	PET1,PET2,PET3,s1,m1,s2,m2,s3,m3,sortElasticityTableBy,sortElasticityTableModes,sortElasticityTableModesCSV,refmodes,i,initializeControls,tabsize},
	    	
	    	initializeControls[resetType_]:=Module[{},
	    			If[resetType == 0, type = 1];
	        		controls = ConstantArray[{},8];
	        		s1 = 1;
	        		m1 = 1;
	        		s2 = 1;
	        		m2 = 1;
	        		s3 = 1;
	        		m3 = 1;
	        		sortElasticityTableBy = 4;
	                ];

		    idx = ddwaDat["mdlLEEA`Private`idx"];
		    behaviorModesAll = ddwaDat["mdlLEEA`Private`behaviorModesAll"];
		    M1 = ddwaDat["mdlLEEA`Private`M1"];
		    Lb = ddwaDat["mdlLEEA`Private`Lb"];
		    WBase = ddwaDat["mdlLEEA`Private`WBase"];
		    PElas = ddwaDat["mdlLEEA`Private`PElas"] /. {Indeterminate -> "Indet."};
		    PInfER = ddwaDat["mdlLEEA`Private`PInfER"];
		    PInfEI = ddwaDat["mdlLEEA`Private`PInfEI"];
		    PElasER = ddwaDat["mdlLEEA`Private`PElasER"] /. {Indeterminate -> "Indet."};
		    PElasEI = ddwaDat["mdlLEEA`Private`PElasEI"] /. {Indeterminate -> "Indet."};
		    ddwaTimePos = ddwaDat["mdlLEEA`Private`ddwaTimePos"];
		    
		    lev = mdlDefinitions["Levels"][[;;,1;;2]];
		    par = #[[1]]->#[[2]]&/@mdlDefinitions["Constants"];
			stocks = Table[i ->  StringDelete[ToString@(mdlDefinitions["Levels"][[i,1]]),"mdl`"], {i, 1, Length[mdlDefinitions["Levels"]]}];
	    	
		    export["Type"] = "CSV";
		    export["ButtonText"] = "Export as CSV";
		    
		    tabsize = {size[[1]]-35,size[[2]]-110};
		    
		    refmodes = Table[behaviorModesAll[[i]] -> 
		    	Switch[Lb[[idx]][[i]],
		    		 _Complex, StringForm["`1`\[PlusMinus]`2``3`", NumberForm[Re[Lb[[idx]][[i]]], {7, 5}], NumberForm[Im[Lb[[idx]][[i]]], {7, 5}], TraditionalForm@I], 
		    		_, StringForm["``",NumberForm[Lb[[idx]][[i]],{7,5}]]
		    	], {i, Length[idx]}];
		    	
	    
		    Dynamic[

	        	If[reset == 1, initializeControls[1],
	        		If[reset == 2, initializeControls[0]]
	    		];
            	reset = 0;

		    	controls[[1]] = type;
		    	controls[[2]] = s1;
		    	controls[[3]] = m1;
		    	controls[[4]] = s2;
		    	controls[[5]] = m2;
		    	controls[[6]] = s3;
		    	controls[[7]] = m3;
		    	controls[[8]] = sortElasticityTableBy;
		    	
				export["FileName"] = If[type == 1, "Stocks & Behavior Modes",
										If[type == 2, "Parameter Elasticity By Stocks",
											If[type == 3, "Parameter Elasticity By Reference Mode",
												"Parameter Elasticity with Eigenvalue Elasticity"
											]
										]
									];
		    	
				timepos = -1;

			    TabView[{{1,"Stocks & Behavior Modes"->
			    	DynamicModule[{},
				    	export["Content"] = {{"Stocks"}, Sequence@@({#[[2]]} & /@ stocks), {" "}, 
				    		{"Behavior Modes"}, {"Number","Re", "Im"}, Sequence@@({#[[1]],Sequence@@#[[2]]} & /@ Table[behaviorModesAll[[i]] -> Switch[Lb[[idx]][[i]], _Complex, {Re[Lb[[idx]][[i]]],Im[Lb[[idx]][[i]]]}, _,{Lb[[idx]][[i]]}], {i, Length[idx]}])};
				    	Pane[
				    		Grid[
				    			{
					    			{
						    			Column[{
						    				Style["Stocks",Bold],
						    				Column[stocks,"\[Rule]"]
						    				},
						    				Dividers->{None,{2->Green}}
						    			],
						    			"",
						    			Column[{
					    					Style["Behavior Modes",Bold],
					    					Column[Thread[(Style[Subscript["\[Lambda]",#],Italic] & /@ refmodes[[;;,1]])->refmodes[[;;,2]]],"\[Rule]"]
				    						},
				    						Dividers->{None,{2->Green}}
			    						]
					    			}	
				    			},
				    		Alignment -> {Center, Top}
				    		],
				    		{tabsize[[1]],tabsize[[2]]+40},
				    		Scrollbars->{False,True},
				    		ImageSizeAction->"Clip",
				    		ImageMargins->{{10,0},{0,10}},
			    			AppearanceElements->None
				    	]
			    	]
			    	},
			    		
(*	    			{2,StringForm["`1` elasticity by Stock", Style["w", Bold, Italic]] ->
			    	Manipulate[
			    		
			    		Module[{table,tableIndex},

							PET1 = Map[Prepend[PElas[[#, All, s1]],StringDelete[ToString@(par[[#]][[1]]),"mdl`"]]&,Range[Length[par]]];
							
							export["Content"] = Prepend[PET1[[Ordering[Abs[PET1[[All,1+m1]]],All,Greater]]], Join[{"Parameter"},
								Table[Switch[Lb[[idx]][[i]],
				    		 			_Complex, ToString@NumberForm[Re[Lb[[idx]][[i]]], {7, 5}] <> " +/- " <> ToString@NumberForm[Im[Lb[[idx]][[i]]], {7, 5}] <> "i", 
				    					_, ToString@NumberForm[Lb[[idx]][[i]],{7,5}]
										],
										{i,Length[idx]}
								]]
							];
							PrependTo[export["Content"],
								{
									"Sorted by Behavior Mode",
									Switch[Lb[[idx]][[m1]],
				    		 			_Complex, ToString@NumberForm[Re[Lb[[idx]][[m1]]], {7, 5}] <> " +/- " <> ToString@NumberForm[Im[Lb[[idx]][[m1]]], {7, 5}] <> "i", 
				    					_, ToString@NumberForm[Lb[[idx]][[m1]],{7,5}]
									]
			    				}
		    				];
							PrependTo[export["Content"],{"Stock",stocks[[s1,2]]}];
							
							table = {
										Join[{Style["Parameter",Bold]},Column[{Style[Subscript["\[Lambda]",refmodes[[#,1]]],Bold,Italic],Style[refmodes[[#,2]],Bold]},Alignment->Center] & /@ Range@Length@behaviorModesAll],
										Sequence@@PET1[[Ordering[Abs[PET1[[All,1+m1]]],All,Greater]]]
									};
							tableIndex = Range[Dimensions[table][[2]]];
							
							Pane[
								Column[{
									Item[
										StringForm["Stock `1`, `2` elasticities sorted by behavior mode `3`", Style[stocks[[s1,2]],Bold], Style[Subscript["w","i"], Bold, Italic], Style[Subscript["\[Lambda]",refmodes[[m1,1]]],Bold,Italic]],
										Alignment->Left
									],
									PaddedForm[
										(*TableForm[PET1[[Ordering[Abs[PET1[[All,1+m1]]],All,Greater]]],
											TableHeadings->{
												None,
												Join[{Style["Parameter",Bold]},Column[{Style[Subscript["\[Lambda]",refmodes[[#,1]]],Bold,Italic],Style[refmodes[[#,2]],Bold]},Alignment->Center] & /@ Range@Length@behaviorModesAll]
											}
										]*)
										GridBox[
											Transpose[Transpose[table][[Insert[Delete[tableIndex, m1+1], tableIndex[[m1+1]],2]]]],
											ColumnAlignments->{Left,Center},
											GridBoxDividers->{"Rows"->{False,True}}
										] // DisplayForm,
										{7,4}
									]
									},
									Dividers->{None,{2->Green}}
								],
								
								tabsize,
								
					    		Scrollbars->{True,True},
					    		ImageSizeAction->"Clip",
				    			AppearanceElements->None
							]
			    		],
			        
					    Row[{
						    "    Stock ",
						    PopupMenu[Dynamic@s1,stocks,FieldSize->Small],
						    " Sorted by Behavior Mode ",
						    PopupMenu[Dynamic@m1,Thread[Range@Length[behaviorModesAll]->(Style[Subscript["\[Lambda]",#],Italic] & /@ refmodes[[;;,1]])],FieldSize->Tiny]
					    },Alignment->Center
					    ],
			        
					    ControlPlacement->Top,
					    TrackedSymbols:>{s1,m1},
					    SynchronousUpdating->True,
					    Paneled->False,
					    AppearanceElements->None
	    			]
		    		},
		    		
		    		{3,StringForm["`1` elasticity by Behavior Mode", Style["w", Bold, Italic]]->
			    	Manipulate[
						
			    		Module[{table,tableIndex},

							PET2 = Map[Prepend[PElas[[#, m2, All]],StringDelete[ToString@(par[[#]][[1]]),"mdl`"]]&,Range[Length[par]]];
		
							export["Content"] = Prepend[PET2[[Ordering[Abs[PET2[[All,1+s2]]],All,Greater]]], Join[{"Parameter"},Table[StringDelete[ToString@(lev[[i]][[1]]),"mdl`"],{i,Length[lev]}]]];
							PrependTo[export["Content"],{"Sorted by Stock",stocks[[s2,2]]}];
							PrependTo[export["Content"],
								{
									"Behavior Mode",
									Switch[Lb[[idx]][[m2]],
				    		 			_Complex, ToString@NumberForm[Re[Lb[[idx]][[m2]]], {7, 5}] <> " +/- " <> ToString@NumberForm[Im[Lb[[idx]][[m2]]], {7, 5}] <> "i", 
				    					_, ToString@NumberForm[Lb[[idx]][[m2]],{7,5}]
									]
			    				}
		    				];

							table = {
										Join[{Style["Parameter",Bold]},Table[Style[StringDelete[ToString@(lev[[i]][[1]]),"mdl`"],Bold],{i,Length[lev]}]],
										Sequence@@PET2[[Ordering[Abs[PET2[[All,1+s2]]],All,Greater]]]
									};
							tableIndex = Range[Dimensions[table][[2]]];
							
							Pane[
								Column[{
									Item[
										StringForm["Behavior mode `1`, `2` elasticities sorted by stock `3`", Style[Subscript["\[Lambda]",refmodes[[m2,1]]],Bold,Italic], Style[Subscript["w",refmodes[[m2,1]]], Bold, Italic], Style[stocks[[s2,2]],Bold] ],
										Alignment->Left
									],
									PaddedForm[
										(*TableForm[
											PET2[[Ordering[Abs[PET2[[All,1+s2]]],All,Greater]]],
											TableHeadings->{None,Join[{"Parameter"},Table[StringDelete[ToString@(lev[[i]][[1]]),"mdl`"],{i,Length[lev]}]]}
										]*)
										GridBox[Transpose[Transpose[table][[Insert[Delete[tableIndex, s2+1], tableIndex[[s2+1]],2]]]],
											ColumnAlignments->{Left,Center},
											GridBoxDividers->{"Rows"->{False,True}}
										] // DisplayForm,
										{7,4}
									]
									},
									Dividers->{None,{2->Green}}
								],
								
								tabsize,
								
					    		Scrollbars->{True,True},
					    		ImageSizeAction->"Clip",
				    			AppearanceElements->None
							]
			    		],
			        
					    Row[{
						    "    Behavior Mode ",
						    PopupMenu[Dynamic@m2,Thread[Range@Length[behaviorModesAll] ->(Style[Subscript["\[Lambda]",#],Italic] & /@ refmodes[[;;,1]])],FieldSize->Tiny],
						    " Sorted by Stock ",
						    PopupMenu[Dynamic@s2,stocks,FieldSize->Small]
					    },Alignment->Center
					    ],
			        
					    ControlPlacement->Top,
					    TrackedSymbols:>{s2,m2},
					    SynchronousUpdating->True,
					    Paneled->False,
					    AppearanceElements->None
	    			]
	    			
		    		},
*)		    		
		    		{4,StringForm["`1` elasticity",Style["\[Lambda]", Bold, Italic]]->
			    	Manipulate[
			    		
			    		sortElasticityTableModes = {
			    				(*1->"Speed",*)
			    				2->"Parameter Name",
					    		3->StringForm["Weight on Mode `1`",Style[#,Italic]],
					    		4->StringForm["Elasticity Re[`1`]",Style[Subscript["\[Lambda]",#],Italic]],
					    		5->StringForm["Elasticity Im[`1`]",Style[Subscript["\[Lambda]",#],Italic]]
					    	} & @ refmodes[[m3,1]];

			    		sortElasticityTableModesCSV = {
			    				(*1->"Speed",*)
			    				2->"Parameter Name",
					    		3->"Weight on Mode " <> ToString@#,
					    		4->"Elasticity Re[Eigenvalue " <> ToString@# <> "]",
					    		5->"Elasticity Im[Eigenvalue " <> ToString@# <> "]"
					    	} & @ refmodes[[m3,1]];
						
						PET3 = Map[Prepend[{PElas[[#,m3,s3]],PElasER[[#,m3]],PElasEI[[#,m3]]},StringDelete[ToString@(par[[#]][[1]]),"mdl`"]]&,Range[Length[par]]];
						
						export["Content"] = Prepend[
												Switch[sortElasticityTableBy,
													1, PET3[[Ordering[Abs[PInfER[[;;,m3]]],All,Greater]]],
													2, PET3[[Ordering[PET3[[All,sortElasticityTableBy-1]]]]],
													_, PET3[[Ordering[Abs[PET3[[All,sortElasticityTableBy-1]]],All,Greater]]]
												], 
												Join[{"Parameter","Weight on Behavior Mode","Re[EV] Elasticity","Im[EV] Elasticity"}]
						];
						PrependTo[export["Content"],{""}];
						PrependTo[export["Content"],
							{
								"Behavior Mode",
								Switch[Lb[[idx]][[m3]],
			    		 			_Complex, ToString@NumberForm[Re[Lb[[idx]][[m3]]], {7, 5}] <> " +/- " <> ToString@NumberForm[Im[Lb[[idx]][[m3]]], {7, 5}] <> "i", 
			    					_, ToString@NumberForm[Lb[[idx]][[m3]],{7,5}]
								],
								"Sorted By",
								Lookup[Association@sortElasticityTableModesCSV, sortElasticityTableBy]
		    				}
	    				];
	    				PrependTo[export["Content"],{""}];
						PrependTo[export["Content"],{"Stock",stocks[[s3,2]]}];
						
						Pane[
							Column[{
								Item[
									StringForm["Stock `1`, `2` elasticities. `3` `4` 0, `5` `6` 0", 
										Style[stocks[[s3,2]],Bold], 
										Style[Subscript["\[Lambda]",refmodes[[m3,1]]],Bold,Italic],
										Style["w",Bold,Italic],
										If[WBase[[m3,s3]] < 0, "<",">"],
										Style["\[Lambda]",Bold,Italic],
										If[Re[Lb[[idx]][[m3]]] < 0, "<",">"]
										],
									Alignment->Left
								],
								PaddedForm[
								(*TableForm[
									PET3[[Ordering[Abs[PET3[[All,2]]],All,Greater]]],
									TableHeadings->{None,Join[{"Parameter","Weight","Re[Eigenvalue]","Im[Eigenvalue]"}]}
								]*)
									GridBox[{
										Style[#,Bold] & /@ 
											Join[
												{"Parameter"},
												Column[{#,"Weight"},Alignment->Center] & /@ 
														{StringForm["Behavior Mode `1`",Style[refmodes[[m3,1]],Italic]]},
												Column[{#,"Elasticity"},Alignment->Center] & /@ 
														{StringForm["Re[`1`]",Style[Subscript["\[Lambda]",refmodes[[m3,1]]],Italic]],StringForm["Im[`1`]",Style[Subscript["\[Lambda]",refmodes[[m3,1]]],Italic]]}
											],
										Switch[sortElasticityTableBy,
													1, Sequence@@PET3[[Ordering[Abs[PInfER[[;;,m3]]],All,Greater]]],
													2, Sequence@@PET3[[Ordering[PET3[[All,sortElasticityTableBy-1]]]]],
													_, Sequence@@PET3[[Ordering[Abs[PET3[[All,sortElasticityTableBy-1]]],All,Greater]]]
										]	
										},
										ColumnAlignments->{Left,Center},
										GridBoxDividers->{"Rows"->{False,True}}
									] // DisplayForm,
									{7,4}
								]
								},
								Dividers->{None,{2->Green}}
							],
							
							tabsize,
							
				    		Scrollbars->{True,True},
				    		ImageSizeAction->"Clip",
			    			AppearanceElements->None
						],
			        
					    Row[{
						    "    Stock ",
						    PopupMenu[Dynamic@s3,stocks,FieldSize->Small],
						    "\tBehavior Mode ",
						    PopupMenu[Dynamic@m3,Thread[Range@Length[behaviorModesAll]->(Style[Subscript["\[Lambda]",#],Italic] & /@ refmodes[[;;,1]])],FieldSize->Tiny],
						    "\tSorted By ",
						    Dynamic@PopupMenu[Dynamic@sortElasticityTableBy, sortElasticityTableModes, FieldSize->Small]
					    },Alignment->Center
					    ],
			        
					    ControlPlacement->Top,
					    TrackedSymbols:>{s3,m3,sortElasticityTableBy},
					    SynchronousUpdating->True,
					    Paneled->False,
					    AppearanceElements->None
	    			]
		    		}
		    		},
			    	Dynamic@type,
			    	FrameMargins -> {{0,0},{0,0}},
			    	ImageSize -> {size[[1]]-10,size[[2]]-20}
			    ],
			    TrackedSymbols :> {type,s1,m1,s2,m2,s3,m3,sortElasticityTableBy,reset},
			    SynchronousUpdating->True,
			    Initialization :> (
								If[ controls=!={},
									type = controls[[1]];
							    	s1 = controls[[2]];
							    	m1 = controls[[3]];
							    	s2 = controls[[4]];
							    	m2 = controls[[5]];
							    	s3 = controls[[6]];
							    	m3 = controls[[7]];
							    	sortElasticityTableBy = controls[[8]]; 
									,
				                 	initializeControls[0];
				                ]
			                )
		    ]
	    ],
	    size-10,
	    FrameMargins->{{0,0},{0,10}},
	    ImageSizeAction->"Clip",
	    AppearanceElements->None
    ]



(* ::Section:: *)
(*mdl Functions*)


(* ::Subsection:: *)
(*Parsing functions*)


SetAttributes[toValidName,Listable];
toValidName[name_] :=
    Module[ {varname},
(*varname=StringDelete[name,RegularExpression["[^[:alnum:]\$]"]]*)
(* delete spaces *)
        varname = StringDelete[name,RegularExpression["\\s*"]];
        (* replace non-alphanumeric characters with $charcode$ *)
        varname = StringReplace[varname,RegularExpression["([^[:alnum:]\$])"]:>"$"<>ToString@First@ToCharacterCode["$1"]<>"$"];
        (* replace first digit with its verbal representation *)
        varname = StringReplace[varname,RegularExpression["^([[:digit:]])(.*)"]:>IntegerName[ToExpression["$1"]]<>"$2"]
    ];

toValidScientificFormat[expr_String] :=
    StringReplace[expr,RegularExpression["([+\\-]?(?:0|[1-9]\\d*)(?:\\.\\d*)?)([eE]([+\\-]?\\d+))"]:>"$1"<>"*^"<>"$3"];

(* vars_: variables to be converted to the MMA compatible names, exprs_: expressions to transform variables and number format *)
toValidExpression[subsRules_List,{}] :=
    {};
toValidExpression[subsRules_List,eqsn_List] :=
    Module[ {exprs},
        exprs = StringReplace[#,subsRules]&/@eqsn;
        exprs = MapAt[toValidScientificFormat,exprs,{;;,2;;}];
        exprs = MapAt[StringDelete[#," "]&,exprs,{;;,2;;}];
        ToExpression[exprs,TraditionalForm]
    ];

toTableFunction[subsRules_List,tFunction_] :=
    Module[ {tfName,tfDataRaw,tfData},
        tfName = tFunction[[1]]/.subsRules;
        tfDataRaw = toValidScientificFormat[tFunction[[2]]];
        tfData = StringCases[tfDataRaw,"("~~x:Except[","]..~~","~~y:Except[","]..~~")":>ToExpression@{x,y}];
        {ToExpression@tfName,Function[ x,
        Evaluate[Piecewise[{{#[[1,2]],x< #[[1,1]]},{Interpolation[Union@#,InterpolationOrder->1][x],#[[1,1]]<= x<= #[[-1,1]]}, {#[[-1,2]],x> #[[-1,1]]}}]&@tfData]]}
    ];


(* ::Subsection:: *)
(*Parser*)


mdlParse::varsubscript = "ERROR: Subscripts are not allowed in this version";
mdlParse::equations = "WARNING: The equation(s) `1` have not been processed";
mdlParse::functions = "ERROR: The function(s) `1` have not been defined";
mdlParse[fileName_] :=
    Module[ {error = False,mdlString,mdlStringEquations,equationsRegex,
           mdlEquations,mdlEquationsNumber,simulationParameters,simulationParametersRegex,mdlParametersRaw,definitionsRegex,constantsRegex,mdlConstantsRaw,levelsRegex,mdlLevelsRaw,
           variablesRegex,mdlVariablesRaw,tableFunctionsRegex,mdlTableFunctionsRaw,dataRegex,mdlDataRaw,variablesRules,contextRules,variablesList,levelsList,tableFunctionsList,dataList,equations,
           functionsReplacement,misssedFunctions},
        Catch[
        Check[
        simulationParameters = {
        "INITIAL TIME",
        "FINAL TIME",
        "SAVEPER",
        "TIME STEP"
        };

        (* read the file as a single string *)
        mdlString = ReadString[fileName];

        (* retrieve the equations definitions part by removing macro and sketch*)
        mdlStringEquations = StringDelete[mdlString,RegularExpression["(?msi)({.*?})|(:macro:.+?:end of macro:)|(\\s*\\\\\\\\\\\\---///.*?(?=(///---\\\\\\\\\\\\)))|(\\s*///---\\\\\\\\\\\\.*)"]];
        (* separate equations *)
        equationsRegex = "(?ms)((?>([^\"~\\|]*(\"(?>\\\\\"|[^\"])+\")?[^\"~\\|]*))+)~(?1)~(?1)(~(?1))?\\|";
        mdlEquations = StringReplace[mdlStringEquations,RegularExpression["\\\\(?=\\r|\\n)"]:>""];
        mdlEquations = StringCases[mdlEquations,RegularExpression[equationsRegex]:>StringDelete["$1",{"\n","\r","\t"}]];
        mdlEquationsNumber = Length@mdlEquations;

        (* extract simulation parameters *)
        simulationParametersRegex[par_] :=
            "(?ms)^"<>par<>"\\s*\\=\\s*(.+)\\s*$";
        mdlParametersRaw = StringCases[mdlEquations,RegularExpression[simulationParametersRegex[#]]:>Sequence[#,"$1"]& /@simulationParameters];
        mdlEquations = Delete[mdlEquations,Position[mdlParametersRaw,Except[{}],{-2}]];
        mdlParametersRaw = Cases[mdlParametersRaw,Except[{}],{1}];
        definitionsRegex = "(?(DEFINE) 
							(?<notallowed>[^()\"~\\|\\:,=\\[\\]])
							(?<number>[\\d\\+\\-eE.]+)
							(?<varname>(?>(?&notallowed)+|\"(?>\\\\\"|[^\"])+\")) 
							
							# variable if subscripts and data variables are allowed
							(?<variablesubs>(?>(\\[\\s*(?&varname)\\s*(,\\s*(?&varname)\\s*)*\\])))
							(?<variabledata>(?>(\\:\\s*(\\w+)\\s*\\:)))
							
							(?<variable>(?>(?&varname))\\s*)
						    (?<equation>(((?&variable)*)(?<parenthesis>\\(((?>(?&variable)+|(?&parenthesis)),?)*\\))?((?&variable)*))+)
						   )";

        (* extract constants *)
        constantsRegex = "(?mx)^((?&variable))=\\s*([\\d\\+\\-eE.]+)\\s*$"<>definitionsRegex;
        mdlConstantsRaw = StringCases[mdlEquations,RegularExpression[constantsRegex]:>Sequence["$1","$2"]];
        mdlEquations = Delete[mdlEquations,Position[mdlConstantsRaw,Except[{}],{-2}]];
        mdlConstantsRaw = Cases[mdlConstantsRaw,Except[{}],{1}];

        (* extract levels *)
        levelsRegex = "(?mx)^((?&variable))=\\s*INTEG\\s*\\(((?&equation)),((?&equation))\\)$"<>definitionsRegex;
        mdlLevelsRaw = StringCases[mdlEquations,RegularExpression[levelsRegex]:>Sequence["$1","$2","$3"]];
        mdlEquations = Delete[mdlEquations,Position[mdlLevelsRaw,Except[{}],{-2}]];
        mdlLevelsRaw = Cases[mdlLevelsRaw,Except[{}],{1}];

        (* extract table functions *)
        tableFunctionsRegex = "(?mx)^((?&variable))\\(\\s*(\\[[^\\]]+\\])?(((\\s*,\\s*)?\\(\\s*(?&number)\\s*,\\s*(?&number)\\s*\\))+)\\s*\\)$"<>definitionsRegex;
        mdlTableFunctionsRaw = StringCases[mdlEquations,RegularExpression[tableFunctionsRegex]:>Sequence["$1","$3"]];
        mdlEquations = Delete[mdlEquations,Position[mdlTableFunctionsRaw,Except[{}],{-2}]];
        mdlTableFunctionsRaw = Cases[mdlTableFunctionsRaw,Except[{}],{1}];

        (* extract data variables *)
        dataRegex = "(?mx)^((?&variable))(?&variabledata)?(\\s*\\:\\=.*)?$"<>definitionsRegex;
        mdlDataRaw = StringCases[mdlEquations,RegularExpression[dataRegex]:>"$1"];
        mdlEquations = Delete[mdlEquations,Position[mdlDataRaw,Except[{}],{-2}]];
        mdlDataRaw = Cases[mdlDataRaw,Except[{}],{1}];

        (* extract variables *)
        variablesRegex = "(?mx)^((?&variable))=(.*)$"<>definitionsRegex;
        mdlVariablesRaw = StringCases[mdlEquations,RegularExpression[variablesRegex]:>Sequence["$1","$2"]];
        mdlEquations = Delete[mdlEquations,Position[mdlVariablesRaw,Except[{}],{-2}]];
        mdlVariablesRaw = Cases[mdlVariablesRaw,Except[{}],{1}];

        (* subscripted variables in the unprocessed equations *)
        If[ Or@@StringContainsQ[mdlEquations,RegularExpression["(?mx)(?&variable)(?&variablesubs)"<>definitionsRegex]],
            Message[mdlParse::varsubscript];
            error = True
        ];

        (* data variables in the unprocessed equations *)
        (* If[Or@@StringContainsQ[mdlEquations,RegularExpression["(?mx)(?&variable)(?&variabledata)?"<>definitionsRegex]],Message[mdlParse::vardata];error=True;];*)
        If[ error,
            Throw[$Failed]
        ];
        If[ mdlEquations=!={},
            Message[mdlParse::equations,ToString@mdlEquations]
        ];

        (* create substitution rules for variable names *)
        variablesRules = Join@@Map[StringTrim[#[[1]]]->toValidName[#[[1]]]&,{mdlParametersRaw,mdlConstantsRaw,mdlVariablesRaw,mdlLevelsRaw, mdlTableFunctionsRaw,mdlDataRaw},{2}];
        variablesRules = SortBy[variablesRules,-StringLength@First@#&];

        (* create substitution rules to add a context *)
        contextRules = ToExpression[#]->ToExpression["mdl`"<>#]&/@variablesRules[[;;,2]];
        AppendTo[contextRules,Symbol["Time"]->mdl`Time];
        contextRules = SortBy[contextRules,-StringLength@ToString@First@#&];
        variablesList = toValidExpression[variablesRules,mdlVariablesRaw];
        levelsList = toValidExpression[variablesRules,mdlLevelsRaw];
        equations = Flatten@{variablesList[[;;,2]],levelsList[[;;,2;;3]]};
        tableFunctionsList = (toTableFunction[variablesRules,#]&/@mdlTableFunctionsRaw);

        (* are there any functions that are neither system nor table functions? If so, capitalize a first letter according to MMA function notation *)
        functionsReplacement = Cases[equations,x_[__]/;Context[x]!="System`"/;!MemberQ[tableFunctionsList[[;;,1]],x]:>x->ToExpression[ToUpperCase[StringTake[ToString@x,1]]<>ToLowerCase[StringTake[ToString@x,2;;]]],Infinity]//Union;
        dataList = toValidExpression[variablesRules,mdlDataRaw];(*toValidName[mdlDataRaw];*)
        variablesList = variablesList//.functionsReplacement;
        levelsList = levelsList//.functionsReplacement;

        (* are there any functions now that are neither system nor table function? *)
        equations = Flatten@{variablesList[[;;,2]],levelsList[[;;,2;;3]]};
        misssedFunctions = Cases[equations,x_[__]/;Context[x]!="System`"/;!MemberQ[tableFunctionsList[[;;,1]],x]:>x,Infinity]//Union;
        misssedFunctions = DeleteCases[misssedFunctions,x_/;MemberQ[tableFunctionsList[[;;,1]],x]];
        If[ misssedFunctions=!={},
            Message[mdlParse::functions,ToString@misssedFunctions];
            Throw[$Failed]
        ];

        (* Return an association of expressions by variable groups *)
        <|"Parameters"->toValidExpression[variablesRules,mdlParametersRaw]//.contextRules,
        "Levels"->levelsList//.contextRules,
        "Variables"->variablesList//.contextRules,
        "Constants"->toValidExpression[variablesRules,mdlConstantsRaw]//.contextRules,
        "Table Functions"->tableFunctionsList//.contextRules,
        "Data"->dataList//.contextRules|>,
        Throw[$Failed]
        ]
        ]
    ];


(* ::Subsection:: *)
(*Solver*)


mdlRun::vardata = "ERROR: Data variables are not allowed in this version";
mdlRun[mdlDefinitions_Association] :=
    Block[ {mdlSolveEuler,tmin,tmax,dt,initialConditions,listLevels,listVariables,levelsEquations,variablesEquations,sol,solcompact,timerange,nsteps,constants},
        Catch[
        Check[
        mdlSolveEuler[] :=
            Block[ {##,mdlEulerStep},
                mdlEulerStep[previousvalues_,currenttime_] :=
                    Block[ {##,levelsCurrent,variablesCurrent,mdl`Time = currenttime},
						(* set current step stocks based on previous step variables *)
                        levelsCurrent = Block[ {##},
                                            MapThread[Set,{listVariables,previousvalues[[2]]}];
                                            MapThread[(#1+dt #2)&,{previousvalues[[1]],levelsEquations}]
                                        ];
                        (* set current step variables based on current step stocks *)
                        MapThread[Set,{listLevels,levelsCurrent}];
                        variablesCurrent = MapThread[Set,{listVariables,variablesEquations}];
                        {levelsCurrent,variablesCurrent}
                    ]&@@(Join@@Lookup[mdlDefinitions,{"Levels","Variables"}][[;;,;;,1]]);

                (* define table functions *)
                (#[[1]][x_] :=
                     #[[2]][x])&/@mdlDefinitions["Table Functions"];

                (* define constant rules *)
                (Evaluate@#[[1]] = #[[2]])&/@mdlDefinitions["Constants"];
                FoldList[mdlEulerStep[#1,#2]&,initialConditions,Range[tmin+dt,tmax,dt]]
            ]&@@(Join@@Values@mdlDefinitions[[;;,;;,1]]);

        (* check if the data varaibles exist *)
        If[ mdlDefinitions["Data"]=!={},
            Message[mdlRun::vardata];
            Throw[$Failed]
        ];

        (* set time variables *)
        tmin = First@Cases[mdlDefinitions["Parameters"],{mdl`INITIALTIME,x_}:>x];
        tmax = First@Cases[mdlDefinitions["Parameters"],{mdl`FINALTIME,x_}:>x];
        dt = First@Cases[mdlDefinitions["Parameters"],{mdl`TIMESTEP,x_}:>x];
        listLevels = mdlDefinitions["Levels"][[;;,1]];
        levelsEquations = mdlDefinitions["Levels"][[;;,2]];
        listVariables = mdlDefinitions["Variables"][[;;,1]];
        variablesEquations = mdlDefinitions["Variables"][[;;,2]];

        (* set initial conditions *)
        initialConditions = Block[ {##,mdl`Time},
                                (Evaluate@#[[1]] = #[[3]])&/@(mdlDefinitions["Levels"]//.mdl`Time->tmin);
                                (Evaluate@#[[1]] = #[[2]])&/@(mdlDefinitions["Variables"]//.mdl`Time->tmin);
                                N@{listLevels,listVariables}
                            ]&@@(Join@@Lookup[mdlDefinitions,{"Levels","Variables"}][[;;,;;,1]]);
        sol = mdlSolveEuler[];
        solcompact = MapThread[SymbolName@#1->#2&,{mdlDefinitions["Levels"][[;;,1]]~Join~mdlDefinitions["Variables"][[;;,1]],DeleteCases[Flatten[sol,{2,3}],{}]}];
        timerange = {"Time"->Range[tmin,tmax,dt]};
        nsteps = Length@sol;
        constants = (SymbolName@#[[1]]->{#[[2]]})&/@mdlDefinitions["Constants"];
        <|timerange,solcompact,constants|>,
        Throw[$Failed]
        ]
        ]
    ];


(* ::Subsection:: *)
(*Interface to LEEA*)


mdlToLEEA::timeVector = "Some or all of the time points `1` are outside of boundaries [`2`, `3`]";
mdlToLEEA[mdlDefinitions_Association,mdlSol_Association,timePoints:_Integer|{__?NumberQ}:10] :=
    Block[ {lev,levInit,var,par,length,solTransposed,tmin, tmax,dt,timeVector,xv,yv,dsv,aux,n,p,imp},
        Catch[
        var = mdlDefinitions["Variables"];
        lev = mdlDefinitions["Levels"][[;;,1;;2]];
        levInit = #[[1]]->#[[3]]&/@mdlDefinitions["Levels"];
        par = #[[1]]->#[[2]]&/@mdlDefinitions["Constants"];
        length = Length@mdlSol["Time"];
        dt = mdlSol["Time"][[2]]-mdlSol["Time"][[1]];
        tmin = mdlSol["Time"][[1]];
        tmax = mdlSol["Time"][[-1]];
        solTransposed = KeyValueMap[Flatten@{"mdl`"<>#1,N@If[ Length[#2]<length,
                                                              Flatten@Table[#2,length],
                                                              #2
                                                          ]}&,mdlSol]//Transpose;
        If[ Head@timePoints==List,
(*            If[ Max@timePoints>tmax||Min@timePoints<tmin,
                Message[mdlToLEEA::timeVector,timePoints,tmin,tmax];
                Throw[$Failed]
            ];*)
            timeVector = Prepend[timePoints+1,1] (*Prepend[Ceiling[(Sort@timePoints-tmin)/dt]+2,1]*),
            Null,
            timeVector = If[ timePoints>length,
                             Range[1,length+1],
                             If[ timePoints==1,
                                 {1,2},
                                 Flatten[{1,Range[2,length,Ceiling[length/(timePoints-1)]],length+1}]//Union
                             ]
                         ];
        ];
        (*ClearAll[xv,yv,dsv,aux,n,p,imp,SubNum];*)
        imp = solTransposed[[timeVector,;;]];
        xv = Map[First,lev];
        yv = Map[First,var];
        dsv = Map[Last,lev];
        aux = Map[Last,var];
        n = Length[xv];
        p = Length[yv];
        AssociationMap[Symbol,{"mdlLEEA`Private`xv","mdlLEEA`Private`yv","mdlLEEA`Private`dsv","mdlLEEA`Private`aux","mdlLEEA`Private`n","mdlLEEA`Private`p","mdlLEEA`Private`imp"}]
        ]
    ]


(* ::Subsection:: *)
(*Check the RMS error with Vensim data file (tab, time across)*)


mdlErrorTotalRMSE[solTAB_,mdlSol_,dp_] :=
    Module[ {},
        Sqrt[Total[((SetAccuracy[solTAB[#],dp]-SetAccuracy[mdlSol[#],dp])^2)&/@Keys[mdlSol],Infinity]]/Total[(Count[mdlSol[#],_,{1}]&/@Keys@mdlSol)]
    ]


(* ::Subsection:: *)
(*Load Vensim .TAB file*)


mdlLoadVensimTAB[vensimfile_,mdlDefs_] :=
    Module[ {vensimData,mdlVensimData,timespos,maintimeseries,sequence,sequenceinterpolate,
    datasplit,datasplittrans,datafunctions,datalabels,datainterpolated},
        vensimData = Import[vensimfile,"TSV"];

        (* check that this is .TAB with time running across *)
        If[ !And@@(MatchQ[#,{_?StringQ,__?NumberQ}]&/@vensimData),
            Return[""]
        ];

        (* interpolate in case there are multiple time axes present due to data variables *)
        timespos = Position[vensimData,{"Time",__}]//Flatten;
        maintimeseries = Rest@vensimData[[First@timespos]];
        sequence = Transpose@{timespos,timespos-1+Differences@Append[timespos,Length@vensimData+1]};
        sequenceinterpolate = Union@Flatten@Rest@Transpose@{timespos+1,timespos-1+Differences@Append[timespos,Length@vensimData+1]};
        datasplit = Rest/@Take[vensimData,#]&/@sequence;
        datasplittrans = Transpose/@Flatten[Thread[{#[[1]],#[[2]]}&@{#[[1]],#[[2;;]]},List,-1]&/@datasplit[[2;;]],1];
        datafunctions = Interpolation[#,InterpolationOrder->1]&/@datasplittrans;
        datalabels = vensimData[[sequenceinterpolate]][[;;,1]];
        datainterpolated = Quiet@MapThread[Flatten@{#1,#2[maintimeseries]}&,{datalabels,datafunctions}];
        vensimData = Take[vensimData,First@sequence]~Join~datainterpolated;

        (* check that this is the correct data file *)
        mdlVensimData = (toValidName[First@#]->Rest@#)& /@ vensimData//Association;
        If[ mdlVensimData["INITIALTIME"]!=Cases[mdlDefs["Parameters"],{mdl`INITIALTIME,x_}:>x] ||
            mdlVensimData["FINALTIME"]!=Cases[mdlDefs["Parameters"],{mdl`FINALTIME,x_}:>x] ||
            mdlVensimData["TIMESTEP"]!=Cases[mdlDefs["Parameters"],{mdl`TIMESTEP,x_}:>x] ||
            Sort["mdl`"<>#1 & /@ Keys@KeyDrop[mdlVensimData,{"Time"}]] =!= Sort[Join@@((ToString/@mdlDefs[#][[;;,1]])&/@Keys@mdlDefs)], (* check that all variables in .TAB are the same as in the model *)
            "",
            KeyDrop[mdlVensimData,{"INITIALTIME","FINALTIME","TIMESTEP","SAVEPER"}]
        ]
    ]


(* ::Subsection:: *)
(*Capture messages*)


SetAttributes[evaluateWithoutMessage,HoldAllComplete];
evaluateWithoutMessage[expr_,myMessageList_,length_] :=
    Internal`InheritedBlock[{Message,$InMsg = False},
    Unprotect[Message];
    Message[msg_,vars___]/;
                        !$InMsg
                        && msg=!=$Off[]
                        && FreeQ[Off/.Internal`QuietStatus[],Unevaluated[msg]]
                        && FreeQ[Stack[], Quiet] :=
        Block[ {$InMsg = True,controlstring},
            controlstring :=
                If[ MatchQ[msg,$Off[_]],
                    msg/.$Off[x_]:>x,
                    First@ReleaseHold[msg/.Cases[HoldPattern@msg,HoldPattern[MessageName[x_,_]]:>Messages@x]]
                ];
            (*With[{messages=ReleaseHold@DeleteDuplicates[#/.HoldPattern@MessageName[s_,_]\[RuleDelayed]Messages@s]&},#/.messages@#]&@msg;*)
            controlstring = If[ StringLength[controlstring]<length,
                                StringPadRight[controlstring,length],
                                controlstring
                            ];
            AppendTo[myMessageList,Style[StringForm[controlstring,vars],Red]]
        (*Message[msg,vars];*)
            (*Print["This is a message: ",vars]*)
            ];
    (*Module[{},Evaluate[expr]]*)
    expr
    ]


(* ::Subsection:: *)
(*Interface*)


mdlDefinitionsQ[mdlDefinitions_] :=
    If[ mdlDefinitions===""||mdlDefinitions===$Failed||Plus@@Length/@Values@mdlDefinitions==0,
        False,
        True
    ];
mdlSolutionQ[mdlSolution_] :=
    If[ mdlSolution===""||mdlSolution===$Failed,
        False,
        True
    ];
mdlLEEAQ[leeaDat_] :=
    If[ leeaDat===""||leeaDat["mdlLEEA`Private`loopsetText"]==={},
        False,
        True
    ];
mdlLEEAInfoQ[exprLEEAInfo_] :=
    If[ exprLEEAInfo==="",
        False,
        True
    ];
mdlSolutionTABQ[mdlSolTAB_] :=
    If[ mdlSolTAB===""||mdlSolTAB===$Failed,
        False,
        True
    ];

mdlDash[] :=
    DynamicModule[{
    		versionSDA = "1.01",

    		resetGrid,resetTimeVector,initializeModelPlot,
    		busy,i,
    		notebookBackground,emptyWindowSize,frameRadius,windowSize,modelParametersWindowsSize,modelFileWindowsSize,modelFileTABWindowsSize,controlWindowSize,plotWindowSize,messageWindowSize,
    		leeaControlWindowSize,leeaControlPanelWindowSize,ddwaSetterBarWindowSize,leeaTimeVectorWindowSize,leeaSetterBarWindowSize,leeaInfoWindowSize,leeaPlotWindowSize,
    		
    		leeaSetterBarWindowTemplate,ddwaSetterBarWindowTemplate,
    		
            interfaceModelFileEmpty,interfaceTABFileEmpty,interfaceModelParametersEmpty,interfaceModelPlotEmpty,
            interfaceLEEAControlEmpty,interfaceLEEASetterBarEmpty,interfaceDDWASetterBarEmpty,interfaceLEEAPlotEmpty,
            
            messageList,spacesize,minStringLength,
            
            fileName,fileNameNew,fileNameTAB,fileNameTABNew,dataSource,
            
            globalNames,

            leaaSetterBarRules,leeaExportData,plotFileName,resetControls,
            
            tmin,tmax,dt,varkeys,levelkeys,firstkey,var,            
            mdlDefs,mdlSol,mdlDat,mdlSolTAB,mdlSolTABNew,exprLEEAInfo,leeaDat,leeaPlot,ddwaDat,ddwaTimePos,ddwaTimePosSelected,leeaRunning,ddwaEnabled,
            leeaControls,leeaDisplay,leeaExportContent,currentTimePos,
            timeList,length,timePoints,maxPoints,defaultMaxPoints,displayMaxPoints,mindt,maxdt,startTime,endTime,intervalTime,timeVector,timeVectorPos,timeVectorLEEA},
             
    resetGrid[] :=
        Module[ {},
            ClearSystemCache[];
            (*Clear[mdlDefs,mdlSol,mdlDat,exprLEEAInfo,leeaDat,leeaPlot,messageList];*)
            
            mdlDefs = "";
            mdlSol = "";
            mdlDat = "";
            mdlSolTAB = "";
            mdlSolTABNew = "";
            exprLEEAInfo = "";
            leeaDat = "";
            leeaPlot = "";
            leeaExportData = <|"ButtonText"->"", "Type"->"", "FileName"->"", "Content"->""|>;
            leeaControls = Table[{},4];
            leeaDisplay = Table["",4];
            leeaExportContent = Table["",4];
            currentTimePos = -1;
            leeaRunning = False;
            ddwaDat = "";
            ddwaTimePos = 0;
            ddwaTimePosSelected = 0;
            ddwaEnabled = False;
            messageList = {};
            tmin = 0;
            tmax = 0;
            dt = 0;
            var = "";
            fileNameTAB = "";
            fileNameTABNew = "";
            dataSource = "";
            resetControls=False;
        ];
   
	resetTimeVector[] :=
		Module[{},
   			timeList = N@mdlSol["Time"];
			length = Length@timeList;
			If[length < defaultMaxPoints, maxPoints = length, maxPoints = defaultMaxPoints];
			
			mindt = dt;
			
			startTime = timeList[[1]];
			endTime = timeList[[-1]];
			intervalTime = If[endTime-startTime < 1, Round[(endTime - startTime)/(maxPoints - 1), dt], Ceiling[(endTime - startTime)/(maxPoints - 1)]];
			timeVector = N@Range[startTime, Min[startTime + intervalTime*(maxPoints - 1), endTime], intervalTime];		
			timeVectorLEEA = timeVector;
		];
		
	initializeModelPlot[] :=
		Module[{},
			levelkeys = StringDelete[ToString/@(mdlDefs["Levels"][[;;,1]]),"mdl`"];
	        varkeys = Sort@StringDelete[ToString/@(mdlDefs["Variables"][[;;,1]]~Join~mdlDefs["Data"][[;;,1]]),"mdl`"];
	        firstkey = If[ levelkeys=!={},
	                       First@levelkeys,
	                       If[ varkeys=!={},
	                           First@varkeys,
	                           ""
	                       ]
	                   ];
			var=firstkey;
		];
        
    resetGrid[];
  
	defaultMaxPoints = 30;
	displayMaxPoints = 15;
    
    windowSize := WindowSize/.Options[EvaluationNotebook[],WindowSize];
    
    modelParametersWindowsSize = {Rasterize[Pane[StringRepeat["x",30]], "RasterSize"][[1]],Rasterize[Pane[Column[{Sequence @@ Table["X", 9]}]], "RasterSize"][[2]] + 10};
    leeaInfoWindowSize = {Rasterize[Pane[StringRepeat["x",30]], "RasterSize"][[1]],Rasterize[Pane[Column[{Sequence @@ Table["X", 9]}]], "RasterSize"][[2]] + 10};
    modelFileWindowsSize = {(modelParametersWindowsSize[[1]]+leeaInfoWindowSize[[1]])/2,Rasterize[Column[{Sequence @@ Table["M", 2]}], "RasterSize"][[2]]};
    modelFileTABWindowsSize = {(modelParametersWindowsSize[[1]]+leeaInfoWindowSize[[1]])/2,Rasterize[Column[{Sequence @@ Table["M", 2]}], "RasterSize"][[2]]};
   	controlWindowSize = {
   		modelParametersWindowsSize[[1]] + leeaInfoWindowSize[[1]] + 20,
   		Max[modelParametersWindowsSize[[2]],leeaInfoWindowSize[[2]]] + Max[modelFileWindowsSize[[2]],modelFileTABWindowsSize[[2]]] + Rasterize[Pane[Column[{Button["M"]}]], "RasterSize"][[2]] + 40
   		}; (* calculate window size based on the content structure *)
    plotWindowSize = {controlWindowSize[[2]]*2.5,controlWindowSize[[2]]};

    leeaTimeVectorWindowSize = Rasterize[
    	Column[{
		    Column[{
		      Item[Style["Time points", Bold], Alignment -> {Center, Center}],
		      Row[{"Start time: ", 
		        InputField[1, Number, FieldSize -> 5, BaseStyle -> LineBreakWithin -> False]}],
		      Row[{" End time: ", InputField[1, Number, FieldSize -> 5, 
		         BaseStyle -> LineBreakWithin -> False]}],
		      Row[{"Interval: ", InputField[1, FieldSize -> 5, BaseStyle -> LineBreakWithin -> False]}]
		      },
		     ":",
		     Dividers -> {None, {-1 -> Dashed, 2 -> Green}},
		     Alignment -> Center
		     ],
						     
		    "",
								    
		    Grid[
		    	Transpose[
		    		{
				   		{
				    	Style["LEEA Time", Bold, TextAlignment -> Center],
			      		Sequence@@Table["M",displayMaxPoints]
				      	},
						
						{
			     		Style["DDWA Time \!\(\*SubscriptBox[\(\[Tau]\), \(0\)]\)", Bold, TextAlignment -> Center],
			     		Sequence@@Table[RadioButton[1,i,Alignment -> Center],{i,1,displayMaxPoints}]	
			     		}
				    }
		     	],
		     	Dividers->{None,{2->Green}},
		     	Spacings->{1,{.5,.5,{0.1}}}
		    ]
		    },
		   	Alignment -> Center
   		], 
  		"RasterSize"
  		] + 10;

    ddwaSetterBarWindowTemplate = 
            Column[Setter[1, #[[1]], #[[2]], Alignment -> Center,ContentPadding -> False] & /@ 
				{
	              1->Style["Decomposition",Italic],
		          1->Style["Elasticity",Italic]
	            }, 
				Spacings -> {0, .5}
			];
    ddwaSetterBarWindowSize = Rasterize[ddwaSetterBarWindowTemplate,"RasterSize"] + 10;

    leeaSetterBarWindowTemplate = 
    		Column[{
                Column[Setter[1, #[[1]], #[[2]], Alignment -> Center,ContentPadding -> False] & /@ 
					{
		              1->Style["Eigenvalues",Italic],
			          1->Style["Loops",Italic]
		            }, 
					Spacings -> {0, 0.5}
				],
	    		"",
	    		Button[" "],
	    		Pane[ddwaSetterBarWindowTemplate,ddwaSetterBarWindowSize],
				"",
				"Output",
	    		Sequence@@Table[Button[" "],3]
    		}];
    leeaSetterBarWindowSize = Rasterize[leeaSetterBarWindowTemplate,"RasterSize"] + 10;
		
    leeaControlPanelWindowSize = {leeaSetterBarWindowSize[[1]], Rasterize[
    		Column[{
	    		Button[" "],
	    		Pane[leeaSetterBarWindowTemplate,leeaSetterBarWindowSize]
    		}],
    	"RasterSize"
		][[2]] + 10};
		
    leeaControlWindowSize = {leeaTimeVectorWindowSize[[1]] + leeaControlPanelWindowSize[[1]] + 30, Max[leeaTimeVectorWindowSize[[2]],leeaControlPanelWindowSize[[2]]] + 20};
	
    messageWindowSize = {controlWindowSize[[1]] + plotWindowSize[[1]] + 10,leeaControlWindowSize[[1]]/3};
    
    leeaPlotWindowSize = {controlWindowSize[[1]] + plotWindowSize[[1]] - leeaControlWindowSize[[1]], leeaControlWindowSize[[2]]};

    frameRadius = 5;
    interfaceModelFileEmpty = Pane[Style["Please load the model",Italic,Green],modelFileWindowsSize,Alignment->{Center,Center}];
    interfaceTABFileEmpty = "";
    interfaceModelParametersEmpty = Pane[Style["The model is not loaded",Italic,Green],modelParametersWindowsSize,Alignment->{Center,Center}];
    interfaceModelPlotEmpty = Pane[Style["Please run the model or load Vensim .TAB file",Italic,Green],plotWindowSize,Alignment->{Center,Center}];
    interfaceLEEAControlEmpty = Pane[Style["Run data is not loaded",Italic,Green],leeaControlWindowSize,Alignment->{Center,Center}];
    interfaceLEEASetterBarEmpty = Pane[Style["Run LEEA",Italic,Green],leeaSetterBarWindowSize,Alignment->{Center,Center}];
    interfaceDDWASetterBarEmpty = Pane[Style["Run DDWA",Italic,Green],ddwaSetterBarWindowSize,Alignment->{Center,Center}];
    interfaceLEEAPlotEmpty = Pane[Style["Please run LEEA",Italic,Green],leeaPlotWindowSize,Alignment->{Center,Center}];
    fileName = "";
    fileNameNew = "";
    emptyWindowSize = Rasterize[Pane[Column@{},ImageSize->{messageWindowSize[[1]],Full},Scrollbars->True,AppearanceElements->None,BaseStyle->LineBreakWithin->False],"RasterSize"];
    spacesize = Rasterize[Column@{" "},"RasterSize"];
    minStringLength = First@Ceiling[emptyWindowSize/spacesize];
    
    leaaSetterBarRules = {
    	(* LEEA *)
		Hold[leeaEigenvaluesOut[mdlDat,leeaDat,leeaPlotWindowSize,leeaExportData,leeaControls[[1]],leeaDisplay[[1]],leeaExportContent[[1]],currentTimePos,resetControls]]->Style["Eigenvalues",Italic],
		Hold[leeaLoopsOut[mdlDat,leeaDat,leeaPlotWindowSize,leeaExportData,leeaControls[[2]],leeaDisplay[[2]],leeaExportContent[[2]],currentTimePos,resetControls]]->Style["Loops",Italic],
		(* DDwWA *)
		Hold[ddwaDDPlot[mdlDefs,mdlDat,ddwaDat,leeaPlotWindowSize,leeaExportData,leeaControls[[3]],currentTimePos,resetControls]]->Style["Decomposition",Italic],
		Hold[ddwaElasticity[mdlDefs,mdlDat,ddwaDat,leeaPlotWindowSize,leeaExportData,leeaControls[[4]],currentTimePos,resetControls]]->Style["Elasticity",Italic]
    };

    
    notebookBackground=CurrentValue[EvaluationNotebook[], Background];
    If[notebookBackground===None,notebookBackground=White];
    
    busy = False;
    
	Dynamic[Overlay[{#,If[busy,ProgressIndicator[Appearance -> "Necklace",ImageSize->100],""]}, All, If[busy,2,1], Alignment -> Center],TrackedSymbols:>{busy},SynchronousUpdating->True] & @
    
    Grid[{
    {
    (* model control and data *)
    Item[
    	Overlay[{
    		Framed[
    			Column[{
    				
				    (* Buttons *)
				    Item[
				    	Row[{
				    		Button["Select Model",
							    fileNameNew = SystemDialogInput["FileOpen",{fileName,{"Vensim models"->{"*.mdl"}}}];
							    If[ fileNameNew=!=$Canceled (*&& fileName!=fileNameNew*),
							        busy = True;

							        (* use new file name and reset the dashboard *)
							        fileName = fileNameNew;
							        resetGrid[];
							    
							    	(* parse and run LEEA Info *)
							        AppendTo[messageList,Style[StringPadRight["Parsing...",minStringLength],Bold]];
							        (* find currently defined symbols *)
							        globalNames = Names["Global`*"];
							        (* parse the model *)
							        evaluateWithoutMessage[mdlDefs = TimeConstrained[mdlParse[fileName],2,$Failed],messageList,minStringLength];
							        (* remove symbols created by mdlParse *)
							        Remove /@ Complement[Names["Global`*"], globalNames];
							        
							        AppendTo[messageList,If[ mdlDefinitionsQ[mdlDefs],
							                                 Style["Parsing complete.",Bold],
							                                 Style["Parsing failed.",Bold,Red]
							                             ]
				                	];
							        If[ mdlDefinitionsQ[mdlDefs],
							            tmin = First@Cases[mdlDefs["Parameters"],{mdl`INITIALTIME,x_}:>x];
							            tmax = First@Cases[mdlDefs["Parameters"],{mdl`FINALTIME,x_}:>x];
							            dt = First@Cases[mdlDefs["Parameters"],{mdl`TIMESTEP,x_}:>x];
							            initializeModelPlot[];
							            AppendTo[messageList,Style[StringPadRight["Structural analysis...",minStringLength],Bold]];
							            evaluateWithoutMessage[exprLEEAInfo = leeaInfo[mdlDefs,messageList],messageList,minStringLength];
							            AppendTo[messageList,If[ mdlLEEAInfoQ[exprLEEAInfo],
							                                     Style["Structural analysis complete.",Bold],
							                                     Style["Structural analysis failed.",Bold,Red]
						                                 		]
				                 		]
						        	];
			                 		busy = False;
						    	],
							    ImageSize->All,
							    Method->"Queued"
						    ],
							    
				    		Style[" then ",Bold],
				    
						    Button["Run Model",
							    (* integrating *)
							    If[ mdlDefinitionsQ[mdlDefs],
							    	busy = True;
							    	
							    	mdlSol="";
							    	dataSource="";
							        AppendTo[messageList,Style["\nIntegrating...",Bold]];
							        evaluateWithoutMessage[mdlSol = mdlRun[mdlDefs];,messageList,minStringLength];
							        AppendTo[messageList,If[ mdlSolutionQ[mdlSol],
							                                 Style["Integration complete.",Bold],
							                                 Style["Integration failed.",Bold,Red]
							                             	]
				             		];
							        If[ mdlSolutionQ[mdlSol],
							            dataSource = Row[{" Data : ",Style["model integration",Bold]}];
										timePoints = maxPoints = defaultMaxPoints;
										resetTimeVector[];
						        	];
						        	fileNameTAB = "";
						        	busy = False;
						    	],
						    	Enabled->Dynamic[fileName =!= "" && mdlDefinitionsQ[mdlDefs] && exprLEEAInfo =!= "" && !mdlLEEAQ[leeaDat]],
						    	Method->"Queued"
					    	],
						    
						    Style[" or ", Bold],
						    
						    (* Load Vensim .TAB file *)
						    Button["Load Vensim Data",
						        fileNameTABNew = SystemDialogInput["FileOpen",{fileNameTAB,{"Vensim TAB files"->{"*.tab"}}}];
						        If[ fileNameTABNew=!=$Canceled,
							    	busy = True;
						            fileNameTAB = fileNameTABNew;
						            AppendTo[messageList,Row[{Style["\nLoading data from ",Bold],FileNameTake[fileNameTAB],Style[" ...",Bold]}]];
						            mdlSolTAB = mdlLoadVensimTAB[fileNameTAB,mdlDefs];
						            AppendTo[messageList,If[ mdlSolutionTABQ[mdlSolTAB],
						                                     Style["Loaded successfully.",Bold],
						                                     Row[{Style["The data file ",Bold,Red],FileNameTake[fileNameTAB],Style[" is invalid or corrupt. Loading failed.",Bold,Red]}]
						                                 	]
				             		];
						            If[ mdlSolutionTABQ[mdlSolTAB],
						                If[ mdlSolutionQ[mdlSol],
						                    dataSource = Column[{
						                                Row[{"Data : ",Style[FileNameTake[fileNameTAB],Bold]}],
						                                Row[{"RMSE Vensim<->integ : ",Style[ScientificForm[mdlErrorTotalRMSE[mdlSolTAB,mdlSol,5],5],Bold] }]
						                                }],
						                    dataSource = Row[{" Data : ",Style[FileNameTake[fileNameTAB],Bold]}]
					                	];
						                mdlSol = mdlSolTAB;
						                tmin = mdlSol["Time"][[1]];
							            tmax = mdlSol["Time"][[-1]];
							            dt = mdlSol["Time"][[2]] - mdlSol["Time"][[1]];
										timePoints = maxPoints = defaultMaxPoints;
										resetTimeVector[];

					            	];
						        	busy = False;
					        	],
						        ImageSize->All,
						        Enabled->Dynamic[fileName =!= "" && mdlDefinitionsQ[mdlDefs] && exprLEEAInfo =!= "" && !mdlLEEAQ[leeaDat]],
						        Method->"Queued"
					        ]
				    
						}],
				    	Alignment->{Center}
					],
    
				    (* Model file name and Vensim TAB file name *)
				    Column[{
				    	Row[{
				    
						    Dynamic[If[ fileName=="",
						                interfaceModelFileEmpty,
						                Pane[
							                Row[{"Model : ",Style[FileNameTake[fileName],Bold]}],
							                modelFileWindowsSize,
							                Alignment->{Center,Center},
							                ImageSizeAction->"ShrinkToFit"
						                	]
						            	],
						            	TrackedSymbols :> {fileName}
						    ],
						    
						    Pane[Dynamic@dataSource,modelFileTABWindowsSize,Alignment->{Center,Center},ImageSizeAction->"ShrinkToFit"]
						    
						}]
						}, 
						Dividers->{None,{-1->Dashed}}
					],
    
				    (* Model parameters and structural info *)
				    Item[
					    Dynamic[If[ !mdlDefinitionsQ[mdlDefs] || !mdlLEEAInfoQ[exprLEEAInfo],
					                interfaceModelParametersEmpty,
					    			Row[{
					    				(* Model info *)
						                Pane[
							                Dynamic@Column[
								                {Style["Model Info",Bold]} ~Join~ 
								                	KeyValueMap[#<>" : "<>ToString[Length[#2]]&,KeyDrop[mdlDefs,"Parameters"]] ~Join~
														{
										                "Initial time : "<>ToString[tmin],
										                "Final time : "<>ToString[tmax],
										                "Time step : "<>ToString[dt]
										                },
							                	":",
							                	Dividers->{None,{2->Green}}
							                ],
						                	modelParametersWindowsSize
					                	],
					    
						    			(* LEEA info *)
						                Pane[
						                	Column[
							                    {
							                    Style["Structural Info",Bold],
							                    "No. of levels : " <> ToString@Length[mdlDefs["Levels"]],
							                    "No. of edges : " <> ToString@Length[exprLEEAInfo["mdlLEEA`Private`el"]],
							                    "No. of strong comp. : " <> ToString@Length[exprLEEAInfo["mdlLEEA`Private`sc"]],
							                    "No. of edges in str. c. : " <> ToString@exprLEEAInfo["mdlLEEA`Private`noesc"],
							                    "No. of nodes : " <> ToString@Length[exprLEEAInfo["mdlLEEA`Private`Mm"]],
							                    "No. of loops : " <> ToString@Length[exprLEEAInfo["mdlLEEA`Private`cl"]],
							                    "No. of geodetic loops : " <> ToString@Length[exprLEEAInfo["mdlLEEA`Private`clg"]],
							                    "No. of indep. fb. loops : " <> ToString[exprLEEAInfo["mdlLEEA`Private`noesc"] - 
							                    	Length[exprLEEAInfo["mdlLEEA`Private`Mm"]] + Length[exprLEEAInfo["mdlLEEA`Private`sc"]]]
							                    },
						                    	":",
						                    	Dividers->{None,{2->Green}}
						                    ],
					                    	leeaInfoWindowSize
					                	]
					    
				    				}]
					    ],
					    TrackedSymbols :> {mdlDefs,exprLEEAInfo}
					    ],
					    Alignment->{Center,Center}
				    ]
    			}],
			    ImageSize->controlWindowSize,
			    RoundingRadius->frameRadius,
			    BaselinePosition->Top->Baseline,
			    FrameMargins->{{5,0},{0,5}}
    		],
    		
    		Row[{" ",Style["Model control",Background->notebookBackground]},BaselinePosition->Center]
    		},
    		
    		All,1,
    		FrameMargins->{{0,0},{0,-10}},
    		ImageMargins->{{0,0},{0,5}}
		],
		
		Alignment->{Left,Center}
	],
    
    SpanFromLeft,
    
    (* Model plots *)
    Item[
    	Overlay[{
		    Dynamic[
		    	Framed[
				    If[ !mdlSolutionQ[mdlSol],
				        interfaceModelPlotEmpty,
				        DynamicModule[{currentPlot,currenttime,plotWidth = plotWindowSize[[1]]-130,plotHeight = plotWindowSize[[2]]-20},
			                       
				            Manipulate[
					        	currenttime = timeVectorLEEA[[Min[Max[1,currentTimePos], Length@timeVectorLEEA]]];
					        	
				                If[ mdlSolutionQ[mdlSol] && var =!= "",
				                	Quiet[currentPlot = ListLinePlot[mdlSol[var],DataRange->{tmin,tmax}, 
				                    						PlotRange->{{timeVectorLEEA[[1]],timeVectorLEEA[[-1]]},All}, 
				                    						PlotRangePadding->{{Scaled[0.007],Automatic},{Automatic,Automatic}}, 
				                                            GridLines->Automatic, 
				                                            GridLinesStyle->Directive[LightGray, Dashed],
				                                            AspectRatio->plotHeight/plotWidth,
				                                            ImageSize->{plotWidth,plotHeight},
				                                            PlotStyle->{Thick,Orange},
				                                            AxesLabel->{"Time",var},
				                                            Epilog->{
				                                            	If[currentTimePos > 0,
				                                            		{Thickness[0.01],Dashed,Red,Line[{{currenttime,Min[mdlSol[var]]},{currenttime,Max[mdlSol[var]]}}]},
				                                            		Nothing
			                                            		]
		                                            		}
				                                        	]
			                    	]
			                	],
				                	
				                Column[{
				                    "\nLevels",
				                    PopupMenu[Dynamic@var,levelkeys,FieldSize->Tiny],
				                    "\nVariables",
				                    PopupMenu[Dynamic@var,varkeys,FieldSize->Tiny],
				                    "",
				                    "",
				                    Button["Export as PDF",
				                        plotFileName = SystemDialogInput["FileSave",{"Model Variable " <> var <> " " <> DateString[DateObject[],{"Month","-","Day","-","Year"," ","Hour","-","Minute","-","Second"}],{"Adobe Acrobat"->{"*.pdf"}}}];
				                        If[ plotFileName=!=$Canceled,
				                            Export[plotFileName,currentPlot,"PDF"]
				                        ],ImageSize->Full,Method->"Queued"
			                    	]
				                    },
				                    Alignment->{Center,Top}
			                   	],
				                
				                TrackedSymbols :> {var,timeVectorLEEA,currentTimePos},
				                SynchronousUpdating->True,
				                Paneled->False,
				                ControlPlacement->Left,
				                AppearanceElements->None,
				                FrameMargins->{{5,0},{0,0}}
				            ]
				        ]
				    ],
				    ImageSize->plotWindowSize,
				    Background->None,
				    RoundingRadius->frameRadius,
				    BaselinePosition->Top->Baseline
		    	],
		    	TrackedSymbols :> {mdlSol}
		    ],
    		Row[{" ",Style["Behavior of model variables",Background->notebookBackground]},BaselinePosition->Center]
    		},
    	    All,1,
    	    FrameMargins->{{0,0},{0,-10}},
    	    ImageMargins->{{0,0},{0,5}}
	    ],
        Alignment->{Left,Center}
    ]
	},
    
    {
    (* LEAA control *)
    Item[
    	Overlay[{
    		Framed[
				Dynamic[
					If[mdlSolutionQ[mdlSol] && mdlDefinitionsQ[mdlDefs],
						
		    			Grid[{{
		    				Dynamic[
								 If[! NumberQ[intervalTime], 
								  intervalTime = timeVector[[2]] - timeVector[[1]]];
								 
								 If[startTime >= endTime - intervalTime, 
								  If[startTime == timeVector[[1]], endTime = startTime + intervalTime,
								    startTime = endTime - intervalTime]];
								 If[endTime > tmax, endTime = tmax];
								 If[startTime < tmin, startTime = tmin];
								 startTime = Round[startTime - tmin, dt] + tmin;
								 endTime = tmax - Round[tmax - endTime, dt];
								 
								 maxdt = Max[dt, Round[(endTime - startTime), dt]];
								 intervalTime = 
								  Round[First@Nearest[Range[mindt, maxdt, dt], intervalTime], dt];
								 timeVector = 
								  N@Range[startTime, 
								   Min[startTime + intervalTime*(maxPoints - 1), endTime], 
								   intervalTime];
							     
							     Module[{vectorPos},
									 vectorPos = 
									  Flatten@Position[timeList, Alternatives @@ timeVector];(*find positions in the times vector*)
									 If[vectorPos =!= timeVectorPos,
									 	timeVectorPos = vectorPos;
									 	ddwaTimePosSelected = 0;
									 	ddwaEnabled = False;
									 ];
							     ];
								 
								 If[(ddwaTimePosSelected =!= ddwaTimePos && ddwaTimePosSelected =!= 0) || (timeVector =!= timeVectorLEEA && ddwaTimePosSelected =!= 0), 
								 	ddwaTimePos = ddwaTimePosSelected; 
								 	ddwaDat = "";
								 	leeaControls[[3;;4]] = {};
						    		If[Or@@(leeaPlot === # & /@ leaaSetterBarRules[[3;;4,1]]), leeaPlot = leaaSetterBarRules[[1,1]]];
							 	];
								 
								 Item[Pane[Column[{
								    Column[{
								      Item[Style["Time points", Bold], Alignment -> {Center, Center}],
								      Row[{"Start time: ", 
								        InputField[Dynamic@startTime, Number, FieldSize -> 5, 
								         BaseStyle -> LineBreakWithin -> False]}],
								      Row[{" End time: ", 
								        InputField[Dynamic@endTime, Number, FieldSize -> 5, 
								         BaseStyle -> LineBreakWithin -> False]}],
								      Row[{"Interval: ", 
								        InputField[Dynamic@intervalTime, 
								         FieldCompletionFunction -> (If[NumberQ@#, 
								             Nearest[Range[mindt, maxdt, dt], Round[#, dt], 3], #] &),
								          FieldSize -> 5, BaseStyle -> LineBreakWithin -> False]}]
								      },
								     ":",
								     Dividers -> {None, {-1 -> Dashed, 2 -> Green}},
								     Alignment -> Center
								     ],
								     
								    "",
								    
								    Grid[
								    	Transpose[
								    		{
										   		{
										    	Style["LEEA Time", Bold, TextAlignment -> Center],
									      		Sequence@@If[Length[timeVector] <= displayMaxPoints,
									      			timeVector,
											      	timeVector[[1;;Floor[displayMaxPoints/2]]] ~Join~ 
											      		{Style["..." <> ToString[Length[timeVector]-(displayMaxPoints-1)] <> " more",Italic]} ~Join~ 
											      		timeVector[[-(displayMaxPoints-Floor[displayMaxPoints/2]-1);;-1]]
									      		]
										      	},
												
												{
									     		Style["DDWA Time \!\(\*SubscriptBox[\(\[Tau]\), \(0\)]\)", Bold, TextAlignment -> Center],
									     		Sequence@@
									     		If[Length[timeVector] <= displayMaxPoints,
													Table[RadioButton[Dynamic@ddwaTimePosSelected,i,Alignment->Center],{i,1,Length[timeVector]}],
													Table[RadioButton[Dynamic@ddwaTimePosSelected,i,Alignment->Center],{i,1,Floor[displayMaxPoints/2]}] ~Join~ 
														{Style["...",Italic]} ~Join~
														Table[RadioButton[Dynamic@ddwaTimePosSelected,i,Alignment->Center],{i,Length[timeVector]+1-(displayMaxPoints-Floor[displayMaxPoints/2]-1),Length[timeVector]}]
												]
									     		(*Table[RadioButton[Dynamic@ddwaTimePos,i,Alignment -> Center],{i,1,Length[timeVector]}]*)	
									     		}
										    }
								     	],
								     	Dividers->{None,{2->Green}},
								     	Spacings->{1,{.5,.5,{0.1}}}
								    ]
								    
								    },
								   	Alignment -> Center
								   ]
								  ,
								  leeaTimeVectorWindowSize, 
								  Alignment -> {Center, Top}
								  ],
								  Alignment -> {Center, Top}
								 ],
								 TrackedSymbols :> {startTime, endTime, intervalTime, ddwaTimePosSelected}
							],
						    
		    				Item[Pane[Column[{
		    					Button["LEEA",
		    						leeaRunning = True;
		    						busy = True;
		    						timeVectorLEEA = timeVector;
		    						currentTimePos = -1;
								    Module[{mdlDatNew = "", leeaDatNew = ""},
									    AppendTo[messageList,Style["\nRunning LEEA...",Bold]];
								    	evaluateWithoutMessage[mdlDatNew = mdlToLEEA[mdlDefs,mdlSol,timeVectorPos],messageList,minStringLength];
								    	If[mdlDatNew =!= "",
									    	evaluateWithoutMessage[leeaDatNew = leeaRun[mdlDefs,mdlDatNew,exprLEEAInfo,messageList],messageList,minStringLength]
								    	];
									    AppendTo[messageList,If[ mdlLEEAQ[leeaDatNew],Style["LEEA complete.",Bold],Style["LEEA failed.",Bold,Red]]];
									    If[mdlDatNew =!= "" && mdlLEEAQ[leeaDatNew],
								    		mdlDat=mdlDatNew;
								    		leeaDat=leeaDatNew;
								    		ddwaDat="";
								    		ddwaEnabled = True;
								    		leeaControls[[3;;4]] = {};
								    		If[Or@@(leeaPlot === # & /@ leaaSetterBarRules[[3;;4,1]]), leeaPlot = leaaSetterBarRules[[1,1]]];
									    ]
								    ];
							    	If[leeaPlot === "",
							    		leeaPlot = leaaSetterBarRules[[1,1]];
							    	];
							    	leeaRunning = False;
							    	busy = False;
							    	leeaDisplay = Table["",4];
							    	leeaExportContent = Table["",4];
								    ,
									Enabled -> Dynamic[fileName!="" && mdlDefinitionsQ[mdlDefs] && mdlSolutionQ[mdlSol]],
									ImageSize -> Full,
									Method -> "Queued"
								],
		    
							    Dynamic[Item[
							    	If[mdlLEEAQ[leeaDat],
							                Pane[

							                	Column[{
							                		
									                Item[Column[Setter[Dynamic@leeaPlot, #[[1]], #[[2]], Alignment -> Center,ContentPadding -> False] & /@ leaaSetterBarRules[[1;;2]],
													 Spacings -> {0, 0.5}
													 ],
													 Alignment -> Center],
											 
							                		"",
							                		
						                			Button["DDWA", 
							                			busy = True;
							                			AppendTo[messageList,Style["\nRunning DDWA...",Bold]];
							                			evaluateWithoutMessage[ddwaDat = ddwaRun[mdlDefs,mdlDat,exprLEEAInfo,leeaDat,ddwaTimePos,messageList],messageList,minStringLength];
							                			AppendTo[messageList,If[ ddwaDat =!= "",Style["DDWA complete.",Bold],Style["DDWA failed.",Bold,Red]]];
							                			If[ddwaDat =!= "",
							                				If[Or@@(leeaPlot === # & /@ leaaSetterBarRules[[1;;2,1]]), 
							                					leeaPlot = leaaSetterBarRules[[3,1]]
							                				]
							                			];
							                			busy = False
							                			,
							                			Enabled -> Dynamic@If[ddwaEnabled && ddwaTimePos != 0, True, False],
							                			ImageSize -> Full, 
							                			Method -> "Queued"
						                			],
						                			
												    Dynamic[Item[
												    	If[ddwaDat =!= "",
											                Pane[
												                Column[Setter[Dynamic@leeaPlot, #[[1]], #[[2]], Alignment -> Center,ContentPadding -> False] & /@ leaaSetterBarRules[[3;;]], 
																 Spacings -> {0, .5}
																 ],
																 ddwaSetterBarWindowSize,
																 Alignment->{Center,Center}
											                ],
											                
											                interfaceDDWASetterBarEmpty
												    	],
												    	Alignment -> Center
												    ]
												    ],
						                			
							                		
							                		"",
												                
									                Item[Style["Output",Bold],Alignment -> Center],
												                
												    Button["Reset current",
												    	resetControls = 1,
												    	Enabled->Dynamic[mdlSolutionQ[mdlSol] && !leeaRunning],
												    	ImageSize->Full
											        	],
											        	
												    Button["Reset all",
												    	resetControls = 2;
					 						            leeaControls = Table[{},4];
					 						            leeaPlot = leaaSetterBarRules[[1,1]];
					 						            ,
												    	Enabled->Dynamic[mdlSolutionQ[mdlSol] && !leeaRunning],
												    	ImageSize->Full
											        	],
										        		
										        	Button[Dynamic[leeaExportData["ButtonText"]],
						                             	plotFileName = SystemDialogInput["FileSave",
									                                {leeaExportData["FileName"] <> ", created " <> DateString[DateObject[],{"Month","-","Day","-","Year"," ","Hour","-","Minute","-","Second"}] <> ".",
									                                    If[ leeaExportData["Type"]=="PDF",
									                                        {"Adobe Acrobat"->{"*.pdf"}},
									                                        {"Comma Separated Values"->{"*.csv"}}
									                                    ]
									                                }
									                                ];
							                            If[ plotFileName=!=$Canceled,
							                            	Block[{header,footer,csvContent={}},
							                            		header = "Model : " <> FileNameTake[fileName];
							                            		footer = "Created with SDA Tool v." <> versionSDA;
								                            	Export[plotFileName,
								                            		If[leeaExportData["Type"] == "PDF",
									                            		Column[{
									                            			Style[header,Gray,Italic,FontFamily -> "Helvetica"],
									                            			"",
									                            			leeaExportData["Content"],
									                            			"",
									                            			Style[footer,10,Gray,Italic,FontFamily -> "Helvetica"]
									                            		}],
									                            		Join[{header},{""},leeaExportData["Content"],{""},{""},{""},{footer}]
								                            		],
								                            		leeaExportData["Type"]
							                            		]
							                            	];
					                            		];
							                            ,
							                            Method->"Queued",
														Enabled->Dynamic@If[mdlSolutionQ[mdlSol] && leeaExportData["Content"] =!= "" && !leeaRunning, True, False],
														ImageSize->Full
										        	]
							                		},
							                		Dividers -> {None, {-4 -> Green}}
							                	],
							                	leeaSetterBarWindowSize,
							                	Alignment->{Center,Center}
						                	],
							                
							                interfaceLEEASetterBarEmpty
							    	],
					               	Alignment->{Center,Center}
		                		],
		                		TrackedSymbols:>{leeaDat}
							    ]
							},
							Alignment->Center
							],
							leeaControlPanelWindowSize,
							Alignment->Center
		    				],
		    				Alignment->{Center,Top}
		    				]
	    				}}
						],
			
						interfaceLEEAControlEmpty
					],
				TrackedSymbols:>{mdlSol,mdlDefs}
				],
				
			    Alignment->{Center,Top},
			    ImageSize->leeaControlWindowSize,
			    RoundingRadius->frameRadius,
			    BaselinePosition->Top->Baseline,
			    FrameMargins->{{0,0},{0,10}}
		    ],
    
			Row[{" ",Style["SDA Control",Background->notebookBackground]},BaselinePosition->Center]
			},
    		All,1,FrameMargins->{{0,0},{0,-10}},ImageMargins->{{0,0},{0,5}}
		],
    
		Alignment->{Left,Center}
	],
    
    (* LEEA output *)
    Dynamic[
	    Item[
	    	Overlay[{
			    Framed[
				    If[ !mdlLEEAQ[leeaDat]||mdlDat==="",
				        interfaceLEEAPlotEmpty,
				        ReleaseHold@leeaPlot
				    ],
			    ImageSize->leeaPlotWindowSize,RoundingRadius->5,BaselinePosition->Top->Baseline
			    ],
			    
			    Row[{" ",Style["SDA output",Background->notebookBackground]},BaselinePosition->Center]
		  	},
		  	All,1,
		  	FrameMargins->{{0,0},{0,-10}},
		  	ImageMargins->{{0,0},{0,5}}
		    ], 
	    Alignment->{Left,Center}
	    ], 
	    TrackedSymbols:>{mdlDat,leeaDat,leeaPlot},
	    SynchronousUpdating -> True
    ],
    
    SpanFromLeft
    
    },
    
    {
    (* message window *)
    Item[
        Overlay[{
            Dynamic@Framed[
                Pane[Column@messageList,{messageWindowSize[[1]]-10,messageWindowSize[[2]]-20},Scrollbars->True,ScrollPosition->{0,1*^15}, AppearanceElements->None],
                ImageSize->messageWindowSize,
                RoundingRadius->frameRadius,
                BaselinePosition->Top->Baseline,
                FrameMargins->{{0,0},{0,10}}
                ],
    Row[{" ",Style["Messages",Background->notebookBackground]},BaselinePosition->Center]
    },All,1,FrameMargins->{{0,0},{0,-10}},ImageMargins->{{0,0},{0,5}}],
    Alignment->{Left,Center}
    ],
    SpanFromLeft,SpanFromLeft
    },
    {Style["Structural Dominance Analysis Toolset v." <> versionSDA <> ". Copyright \[Copyright] 2017 Sergey Naumov (snaumov@sloan.mit.edu) and Rogelio Oliva (roliva@tamu.edu). All Rights Reserved."],SpanFromLeft,SpanFromLeft}
    },
    ItemSize -> Full, 
    BaseStyle->LineBreakWithin->False,
    Frame->None
    ]
    ]
    



(* ::Section:: *)
(*Package ends*)


End[];


Protect[];


EndPackage[]
